# -*- coding: utf-8 -*-
from itertools import chain

import pandas as pd
from tigeropen.common.response import TigerResponse

COLUMNS = ['symbol', 'reportDate', 'reportTime', 'executeDate', 'expectedEps', 'actualEps',
           'fiscalQuarterEnding', 'market', 'exchange', 'actionType']
EARNINGS_CALENDAR_FIELD_MAPPINGS = {'actionType': 'action_type', 'actualEps': 'actual_eps',
                                    'expectedEps': 'expected_eps', 'executeDate': 'execute_date',
                                    'fiscalQuarterEnding': 'fiscal_quarter_ending', 'reportDate': 'report_date',
                                    'reportTime': 'report_time',
                                    }


class EarningsCalendarResponse(TigerResponse):
    def __init__(self):
        super(EarningsCalendarResponse, self).__init__()
        self.earnings_calendar = None
        self._is_success = None

    def parse_response_content(self, response_content):
        response = super(EarningsCalendarResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']

        if self.data:
            self.earnings_calendar = pd.DataFrame(chain.from_iterable(self.data.values()), columns=COLUMNS).rename(
                columns=EARNINGS_CALENDAR_FIELD_MAPPINGS).sort_values(by=['report_date']).reset_index(drop=True)



# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""
from tigeropen.common.consts import Language
from tigeropen.tiger_open_config import TigerOpenClientConfig
from tigeropen.common.util.signature_utils import read_private_key


def get_client_config():
    """
    https://www.itiger.com/openapi/info 开发者信息获取
    :return:
    """
    is_sandbox = False
    client_config = TigerOpenClientConfig(sandbox_debug=is_sandbox)
    client_config.private_key = read_private_key('your private key file path')
    client_config.tiger_id = 'your tiger id'
    client_config.account = 'your account'  # 账户. 不论是环球账户, 综合账户或是模拟账户, 都填在此处
    # standard_account 属性和 paper_account 属性只是为多账户时取用方便, 一般可忽略
    client_config.standard_account = None
    client_config.paper_account = None
    client_config.secret_key = None  # 机构交易员专有密钥 (机构用户需要填写, 个人开发者无需填写)
    client_config.language = Language.en_US
    return client_config



# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""

CONTRACT_FIELDS = set(['symbol', 'market', 'multiplier', 'sec_type', 'currency', 'local_symbol', 'origin_symbol',
                       'expiry', 'strike', 'right', 'contract_id', 'exchange', 'name', 'short_margin', 'short_fee_rate',
                       'shortable', 'long_initial_margin', 'long_maintenance_margin', 'contract_month', 'identifier',
                       'primary_exchange', 'min_tick', 'trading_class', 'status', 'continuous', 'trade',
                       'last_trading_date', 'first_notice_date', 'last_bidding_close_time'])




# -*- coding: utf-8 -*-
"""
Created on 2018/9/20

@author: gaoan
"""
from tigeropen.common.consts import THREAD_LOCAL, SecurityType, Market, Currency
from tigeropen.trade.domain.order import Order
from tigeropen.trade.response.account_profile_response import ProfilesResponse

from tigeropen.trade.response.contracts_response import ContractsResponse
from tigeropen.trade.response.order_id_response import OrderIdResponse
from tigeropen.trade.response.order_preview_response import PreviewOrderResponse
from tigeropen.trade.response.orders_response import OrdersResponse
from tigeropen.tiger_open_client import TigerOpenClient, ApiException
from tigeropen.trade.request.model import ContractParams, AccountsParams, AssetParams, PositionParams, OrdersParams, \
    OrderParams, PlaceModifyOrderParams, CancelOrderParams
from tigeropen.quote.request import OpenApiRequest
from tigeropen.trade.response.assets_response import AssetsResponse
from tigeropen.common.consts.service_types import CONTRACTS, ACCOUNTS, POSITIONS, ASSETS, ORDERS, ORDER_NO, \
    CANCEL_ORDER, MODIFY_ORDER, PLACE_ORDER, ACTIVE_ORDERS, INACTIVE_ORDERS, FILLED_ORDERS, CONTRACT, PREVIEW_ORDER

import logging

from tigeropen.trade.response.positions_response import PositionsResponse


class TradeClient(TigerOpenClient):
    def __init__(self, client_config, logger=None):
        if not logger:
            logger = logging.getLogger('tiger_openapi')
        super(TradeClient, self).__init__(client_config, logger=logger)
        if client_config:
            self._account = client_config.account
            self._standard_account = client_config.standard_account
            self._paper_account = client_config.paper_account
            self._lang = client_config.language
            self._secret_key = client_config.secret_key
        else:
            self._account = None
            self._standard_account = None
            self._paper_account = None
            self._secret_key = None

    def get_managed_accounts(self, account=None):
        """
        获取管理的账号列表
        :param account:
        :return: AccountProfile 对象, 有如下属性：
            account： 交易账户
            capability： 账户类型(CASH:现金账户, MGRN: Reg T 保证金账户, PMGRN: 投资组合保证金)
            status： 账户状态(New, Funded, Open, Pending, Abandoned, Rejected, Closed, Unknown)
        """
        params = AccountsParams()
        params.account = account if account else self._account
        request = OpenApiRequest(ACCOUNTS, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = ProfilesResponse()
            response.parse_response_content(response_content)
            if response.is_success():
                return response.profiles
            else:
                raise ApiException(response.code, response.message)
        return None

    def get_contracts(self, symbol, sec_type=SecurityType.STK, currency=None, exchange=None):
        """
        批量获取合约
        :param symbol:
        :param sec_type: 合约类型 tigeropen.common.consts.SecurityType
        :param currency: 币种 tigeropen.common.consts.Currency
        :param exchange: 交易所
        :return: 合约对象列表, 每个列表项的对象信息同 get_contract 返回
        """
        params = ContractParams()
        params.account = self._account
        params.secret_key = self._secret_key
        params.symbols = symbol if isinstance(symbol, list) else [symbol]
        if sec_type:
            params.sec_type = sec_type.value
        if currency:
            params.currency = currency.value
        params.exchange = exchange

        request = OpenApiRequest(CONTRACTS, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = ContractsResponse()
            response.parse_response_content(response_content)
            if response.is_success():
                return response.contracts
            else:
                raise ApiException(response.code, response.message)

        return None

    def get_contract(self, symbol, sec_type=SecurityType.STK, currency=None, exchange=None, expiry=None, strike=None,
                     put_call=None):
        """
        获取合约
        :param symbol:
        :param sec_type: 合约类型 tigeropen.common.consts.SecurityType
        :param currency: 币种 tigeropen.common.consts.Currency
        :param exchange: 交易所
        :param expiry: 合约到期日(期货/期权) yyyyMMdd
        :param strike: 行权价(期权)
        :param put_call: CALL/PUT
        :return: Contract 对象. 有如下属性:
            symbol: 合约 symbol
            identifier: 合约唯一标识
            currency: 币种
            exchange: 交易所
            name: 合约名称
            sec_type: 合约类型
            long_initial_margin: 做多初始保证金比例
            long_maintenance_margin: 做多维持保证金比例
            short_fee_rate: 做空费率
            short_margin: 做空保证金
            shortable: 做空池剩余
            multiplier: 合约乘数
            expiry: 合约到期日(期货/期权)
            contract_month: 合约月份(期货)
            strike: 行权价(期权)
            put_call: 看跌/看涨(期权)
        """
        params = ContractParams()
        params.account = self._account
        params.secret_key = self._secret_key
        params.symbol = symbol
        if sec_type:
            params.sec_type = sec_type.value
        if currency:
            params.currency = currency.value
        if expiry:
            params.expiry = expiry
        if strike:
            params.strike = strike
        if put_call:
            params.right = put_call
        params.exchange = exchange

        request = OpenApiRequest(CONTRACT, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = ContractsResponse()
            response.parse_response_content(response_content)
            if response.is_success():
                return response.contracts[0] if len(response.contracts) == 1 else None
            else:
                raise ApiException(response.code, response.message)

        return None

    def get_positions(self, account=None, sec_type=SecurityType.STK, currency=Currency.ALL, market=Market.ALL,
                      symbol=None, sub_accounts=None):
        """
        获取持仓数据
        :param account:
        :param sec_type:
        :param currency:
        :param market:
        :param symbol:
        :param sub_accounts:
        :return: 由 Position 对象构成的列表. Position 对象有如下属性:
            account: 所属账户
            contract: 合约对象
            quantity: 持仓数量
            average_cost: 持仓成本
            market_price: 最新价格
            market_value: 市值
            realized_pnl: 实现盈亏
            unrealized_pnl: 持仓盈亏
        """
        params = PositionParams()
        params.account = account if account else self._account
        params.secret_key = self._secret_key
        if sec_type:
            params.sec_type = sec_type.value
        params.sub_accounts = sub_accounts
        if currency:
            params.currency = currency.value
        if market:
            params.market = market.value
        params.symbol = symbol

        request = OpenApiRequest(POSITIONS, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = PositionsResponse()
            response.parse_response_content(response_content)
            if response.is_success():
                return response.positions
            else:
                raise ApiException(response.code, response.message)

        return None

    def get_assets(self, account=None, sub_accounts=None, segment=False, market_value=False):
        """
        获取账户资产信息
        :param account:
        :param sub_accounts: 子账户列表
        :param segment: 是否包含证券/期货分类
        :param market_value: 是否包含分市场市值
        :return: 由 PortfolioAccount 对象构成的列表. PortfolioAccount 对象下的 summary 属性包含一个 Account 对象，
         Account 对象有如下属性：
            net_liquidation: 净清算值
            accrued_cash: 净累计利息
            accrued_dividend: 净累计分红
            available_funds: 可用资金(可用于交易)
            accrued_interest: 累计利息
            buying_power: 购买力
            cash: 证券账户金额+期货账户金额
            currency: 货币
            cushion: 当前保证金缓存
            day_trades_remaining: 剩余日内交易次数，-1表示无限制
            equity_with_loan: 含借贷值股权
            excess_liquidity: 当前结余流动性，为保持当前拥有的头寸，必须维持的缓冲保证金的数额，日内风险数值（App）
            gross_position_value: 持仓市值
            initial_margin_requirement: 初始保证金要求
            maintenance_margin_requirement: 维持保证金要求
            regt_equity: RegT 资产
            regt_margin: RegT 保证金
            sma: 特殊备忘录账户，隔夜风险数值（App）
            settled_cash: 结算利息
            leverage: 总杠杆
            net_leverage: 净杠杆
        """
        params = AssetParams()
        params.account = account if account else self._account
        params.secret_key = self._secret_key
        params.sub_accounts = sub_accounts
        params.segment = segment
        params.market_value = market_value

        request = OpenApiRequest(ASSETS, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = AssetsResponse()
            response.parse_response_content(response_content)
            if response.is_success():
                return response.assets
            else:
                raise ApiException(response.code, response.message)

        return None

    def get_orders(self, account=None, sec_type=None, market=Market.ALL, symbol=None, start_time=None, end_time=None,
                   limit=100, is_brief=False, states=None):
        """
        获取订单列表
        :param account:
        :param sec_type:
        :param market:
        :param symbol:
        :param start_time: 开始时间. 若是时间戳需要精确到毫秒, 为13位整数；
                                    或是日期时间格式的字符串，如"2017-01-01"和 "2017-01-01 12:00:00"
        :param end_time: 截至时间. 格式同 start_time
        :param limit: 每次获取订单的数量
        :param is_brief: 是否返回精简的订单数据
        :param states: 订单状态枚举对象列表, 可选, 若传递则按状态筛选
        :return: Order 对象构成的列表. Order 对象信息参见 tigeropen.trade.domain.order
        """
        params = OrdersParams()
        params.account = account if account else self._account
        params.secret_key = self._secret_key
        if sec_type:
            params.sec_type = sec_type.value
        params.market = market.value
        params.symbol = symbol
        params.start_date = start_time
        params.end_date = end_time
        params.limit = limit
        params.is_brief = is_brief
        params.states = [state.value for state in states] if states else None
        request = OpenApiRequest(ORDERS, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = OrdersResponse()
            response.parse_response_content(response_content, secret_key=params.secret_key)
            if response.is_success():
                return response.orders
            else:
                raise ApiException(response.code, response.message)
        return None

    def get_open_orders(self, account=None, sec_type=None, market=Market.ALL, symbol=None, start_time=None,
                        end_time=None, parent_id=None):
        """
        获取待成交订单列表. 参数同 get_orders
        :param parent_id: 主订单 order_id
        """
        params = OrdersParams()
        params.account = account if account else self._account
        params.secret_key = self._secret_key
        if sec_type:
            params.sec_type = sec_type.value
        params.market = market.value
        params.symbol = symbol
        params.start_date = start_time
        params.end_date = end_time
        params.parent_id = parent_id
        request = OpenApiRequest(ACTIVE_ORDERS, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = OrdersResponse()
            response.parse_response_content(response_content, secret_key=params.secret_key)
            if response.is_success():
                return response.orders
            else:
                raise ApiException(response.code, response.message)
        return None

    def get_cancelled_orders(self, account=None, sec_type=None, market=Market.ALL, symbol=None, start_time=None,
                             end_time=None):
        """
        获取已撤销订单列表. 参数同 get_orders
        """
        params = OrdersParams()
        params.account = account if account else self._account
        params.secret_key = self._secret_key
        if sec_type:
            params.sec_type = sec_type.value
        params.market = market.value
        params.symbol = symbol
        params.start_date = start_time
        params.end_date = end_time
        request = OpenApiRequest(INACTIVE_ORDERS, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = OrdersResponse()
            response.parse_response_content(response_content, secret_key=params.secret_key)
            if response.is_success():
                return response.orders
            else:
                raise ApiException(response.code, response.message)
        return None

    def get_filled_orders(self, account=None, sec_type=None, market=Market.ALL, symbol=None, start_time=None,
                          end_time=None):
        """
        获取已成交订单列表. 参数同 get_orders
        """
        params = OrdersParams()
        params.account = account if account else self._account
        params.secret_key = self._secret_key
        if sec_type:
            params.sec_type = sec_type.value
        params.market = market.value
        params.symbol = symbol
        params.start_date = start_time
        params.end_date = end_time
        request = OpenApiRequest(FILLED_ORDERS, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = OrdersResponse()
            response.parse_response_content(response_content, secret_key=params.secret_key)
            if response.is_success():
                return response.orders
            else:
                raise ApiException(response.code, response.message)
        return None

    def get_order(self, account=None, id=None, order_id=None, is_brief=False):
        """
        获取指定订单
        :param account:
        :param id:
        :param order_id:
        :param is_brief: 是否返回精简的订单数据
        :return: Order 对象. 对象信息参见 tigeropen.trade.domain.order
        """
        params = OrderParams()
        params.account = account if account else self._account
        params.secret_key = self._secret_key
        params.id = id
        params.order_id = order_id
        params.is_brief = is_brief
        request = OpenApiRequest(ORDERS, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = OrdersResponse()
            response.parse_response_content(response_content, secret_key=params.secret_key)
            if response.is_success():
                return response.orders[0] if len(response.orders) == 1 else None
            else:
                raise ApiException(response.code, response.message)
        return None

    def create_order(self, account, contract, action, order_type, quantity, limit_price=None, aux_price=None,
                     trail_stop_price=None, trailing_percent=None, percent_offset=None, time_in_force=None,
                     outside_rth=None, order_legs=None, algo_params=None):
        """
        创建订单对象.
        :param account:
        :param contract:
        :param action:
        :param order_type:
        :param quantity:
        :param limit_price: 限价
        :param aux_price: 在止损单表示止损价格; 在跟踪止损单表示价差
        :param trail_stop_price: 跟踪止损单--触发止损单的价格
        :param trailing_percent: 跟踪止损单--百分比
        :param percent_offset:
        :param time_in_force: 订单有效期， 'DAY'（当日有效）和'GTC'（取消前有效)
        :param outside_rth: 是否允许盘前盘后交易(美股专属)
        :param order_legs: 附加订单
        :param algo_params: 算法订单参数
        """
        params = AccountsParams()
        params.account = account if account else self._account
        params.secret_key = self._secret_key
        request = OpenApiRequest(ORDER_NO, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = OrderIdResponse()
            response.parse_response_content(response_content)
            if response.is_success():
                order_id = response.order_id
                order = Order(account, contract, action, order_type, quantity, limit_price=limit_price,
                              aux_price=aux_price, trail_stop_price=trail_stop_price,
                              trailing_percent=trailing_percent, percent_offset=percent_offset,
                              time_in_force=time_in_force, outside_rth=outside_rth, order_id=order_id,
                              order_legs=order_legs, algo_params=algo_params, secret_key=params.secret_key)
                return order
            else:
                raise ApiException(response.code, response.message)

        return None

    def preview_order(self, order):
        """
        预览订单
        :param order:  Order 对象
        :return: dict. 字段如下
            init_margin_before      下单前账户初始保证金
            init_margin             预计下单后的账户初始保证金
            maint_margin_before     下单前账户的维持保证金
            maint_margin            预计下单后的账户维持保证金
            margin_currency         保证金货币币种
            equity_with_loan_before 下单前账户的含借贷值股权(含贷款价值资产)
            equity_with_loan        下单后账户的含借贷值股权(含贷款价值资产)
            min_commission          预期最低佣金
            max_commission          预期最高佣金
            commission_currency     佣金货币币种

            若无法下单, 返回的 dict 中仅有如下字段:
            warning_text            无法下单的原因
        """
        params = PlaceModifyOrderParams()
        params.account = order.account
        params.contract = order.contract
        params.action = order.action
        params.order_type = order.order_type
        params.order_id = order.order_id
        params.quantity = order.quantity
        params.limit_price = order.limit_price
        params.aux_price = order.aux_price
        params.trail_stop_price = order.trail_stop_price
        params.trailing_percent = order.trailing_percent
        params.percent_offset = order.percent_offset
        params.time_in_force = order.time_in_force
        params.outside_rth = order.outside_rth
        params.secret_key = order.secret_key if order.secret_key else self._secret_key
        request = OpenApiRequest(PREVIEW_ORDER, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = PreviewOrderResponse()
            response.parse_response_content(response_content)
            if response.is_success():
                return response.preview_order
            else:
                raise ApiException(response.code, response.message)

    def place_order(self, order):
        """
        下单
        :param order:  Order 对象
        :return:
        """
        params = PlaceModifyOrderParams()
        params.account = order.account
        params.contract = order.contract
        params.action = order.action
        params.order_type = order.order_type
        params.order_id = order.order_id
        params.quantity = order.quantity
        params.limit_price = order.limit_price
        params.aux_price = order.aux_price
        params.trail_stop_price = order.trail_stop_price
        params.trailing_percent = order.trailing_percent
        params.percent_offset = order.percent_offset
        params.time_in_force = order.time_in_force
        params.outside_rth = order.outside_rth
        params.order_legs = order.order_legs
        params.algo_params = order.algo_params
        params.secret_key = order.secret_key if order.secret_key else self._secret_key

        request = OpenApiRequest(PLACE_ORDER, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = OrderIdResponse()
            response.parse_response_content(response_content)
            if response.is_success():
                order.id = response.id
                if order.order_id:
                    return response.order_id == order.order_id
                else:
                    order.order_id = response.order_id
                    return True
            else:
                raise ApiException(response.code, response.message)

        return False

    def modify_order(self, order, quantity=None, limit_price=None, aux_price=None,
                     trail_stop_price=None, trailing_percent=None, percent_offset=None,
                     time_in_force=None, outside_rth=None):
        """
        修改订单
        :param order:
        :param quantity:
        :param limit_price: 限价
        :param aux_price: 在止损单表示止损价格; 在跟踪止损单表示价差
        :param trail_stop_price: 跟踪止损单--触发止损单的价格
        :param trailing_percent: 跟踪止损单--百分比
        :param percent_offset:
        :param time_in_force: 订单有效期， 'DAY'（当日有效）和'GTC'（取消前有效)
        :param outside_rth: 是否允许盘前盘后交易(美股专属)
        :return:
        """
        params = PlaceModifyOrderParams()
        params.account = order.account
        params.order_id = order.order_id
        params.id = order.id
        params.contract = order.contract
        params.action = order.action
        params.order_type = order.order_type
        params.quantity = quantity if quantity is not None else order.quantity
        params.limit_price = limit_price if limit_price is not None else order.limit_price
        params.aux_price = aux_price if aux_price is not None else order.aux_price
        params.trail_stop_price = trail_stop_price if trail_stop_price is not None else order.trail_stop_price
        params.trailing_percent = trailing_percent if trailing_percent is not None else order.trailing_percent
        params.percent_offset = percent_offset if percent_offset is not None else order.percent_offset
        params.time_in_force = time_in_force if time_in_force is not None else order.time_in_force
        params.outside_rth = outside_rth if outside_rth is not None else order.outside_rth
        params.secret_key = order.secret_key if order.secret_key else self._secret_key

        request = OpenApiRequest(MODIFY_ORDER, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = OrderIdResponse()
            response.parse_response_content(response_content)
            if response.is_success():
                return response.order_id == order.order_id if order.order_id else response.id == order.id
            else:
                raise ApiException(response.code, response.message)

        return False

    def cancel_order(self, account=None, id=None, order_id=None):
        """
        取消订单
        :param account:
        :param id: 全局订单 id
        :param order_id: 账户自增订单 id
        :return:
        """
        params = CancelOrderParams()
        params.account = account if account else self._account
        params.secret_key = self._secret_key
        params.order_id = order_id
        params.id = id
        request = OpenApiRequest(CANCEL_ORDER, biz_model=params)
        response_content = self.__fetch_data(request)
        if response_content:
            response = OrderIdResponse()
            response.parse_response_content(response_content)
            if response.is_success():
                return response.order_id == order_id if order_id else response.id == id
            else:
                raise ApiException(response.code, response.message)

        return False

    def __fetch_data(self, request):
        try:
            response = super(TradeClient, self).execute(request)
            return response
        except Exception as e:
            if THREAD_LOCAL.logger:
                THREAD_LOCAL.logger.error(e, exc_info=True)
            raise e



# -*- coding: utf-8 -*-
"""
Created on 2018/9/20

@author: gaoan
"""
from tigeropen.common.consts import SecurityType


class Contract(object):
    def __init__(self, symbol, currency, contract_id=None, sec_type=None, exchange=None, origin_symbol=None,
                 local_symbol=None, expiry=None, strike=None, put_call=None, multiplier=None, name=None,
                 short_margin=None, short_fee_rate=None, shortable=None, long_initial_margin=None,
                 long_maintenance_margin=None, contract_month=None, identifier=None, primary_exchange=None,
                 market=None, min_tick=None, trading_class=None, status=None, continuous=None, trade=None,
                 last_trading_date=None, first_notice_date=None, last_bidding_close_time=None):
        self.contract_id = contract_id
        self.symbol = symbol
        self.currency = currency
        self.sec_type = sec_type
        self.exchange = exchange
        self.origin_symbol = origin_symbol
        self.local_symbol = local_symbol
        # 到期日
        self.expiry = expiry
        # 行权价
        self.strike = strike
        # 看跌/看涨
        self.put_call = put_call
        self.right = self.put_call
        # 合约乘数
        self.multiplier = multiplier
        # 合约名称
        self.name = name
        # 做空保证金比例
        self.short_margin = short_margin
        # 做空费率
        self.short_fee_rate = short_fee_rate
        # 做空池剩余
        self.shortable = shortable
        # 做多初始保证金
        self.long_initial_margin = long_initial_margin
        # 做多维持保证金
        self.long_maintenance_margin = long_maintenance_margin
        # 合约月份
        self.contract_month = contract_month
        # 合约标识符
        self.identifier = identifier
        # 股票上市交易所
        self.primary_exchange = primary_exchange
        # 市场
        self.market = market
        # 最小报价单位
        self.min_tick = min_tick
        # 合约的交易级别名称
        self.trading_class = trading_class
        # 状态
        self.status = status
        # 期货专有，是否连续合约
        self.continuous = continuous
        # 期货专有，是否可交易
        self.trade = trade
        # 期货专有，最后交易日
        self.last_trading_date = last_trading_date
        # 期货专有，第一通知日，合约在第一通知日后无法开多仓. 已有的多仓会在第一通知日之前（通常为前三个交易日）被强制平仓
        self.first_notice_date = first_notice_date
        # 期货专有，竞价截止时间
        self.last_bidding_close_time = last_bidding_close_time

    def __repr__(self):
        identifier = self.identifier if self.identifier else self.symbol
        if self.sec_type == SecurityType.FUT.value:
            return '%s/%s/%s/%s' % (identifier, self.sec_type, self.currency, self.exchange)
        else:
            return '%s/%s/%s' % (identifier, self.sec_type, self.currency)

    def is_cn_stock(self):
        """
        是否是A股
        :return:
        """
        return self.sec_type == 'STK' and (self.currency == 'CNH' or self.currency == 'CNY')



# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""
import json
import six
import pandas as pd
from tigeropen.common.consts import TradingSession
from tigeropen.common.util.string_utils import get_string
from tigeropen.quote.domain.quote_brief import HourTrading
from tigeropen.common.response import TigerResponse

COLUMNS = ['time', 'price', 'avg_price', 'pre_close', 'volume']
TIMELINE_FIELD_MAPPINGS = {'avgPrice': 'avg_price'}
BRIEF_FIELD_MAPPINGS = {'open': 'open_price', 'high': 'high_price', 'low': 'low_price', 'preClose': 'prev_close',
                        'latestPrice': 'latest_price'}


class QuoteHourTradingTimelineResponse(TigerResponse):
    def __init__(self):
        super(QuoteHourTradingTimelineResponse, self).__init__()
        self.timelines = []
        self.hour_trading = None
        self._is_success = None

    def parse_response_content(self, response_content):
        response = super(QuoteHourTradingTimelineResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']

        if self.data:
            data_json = json.loads(self.data)
            pre_close = data_json.get('preClose')
            if 'detail' in data_json:
                detail = data_json['detail']
                hour_trading = HourTrading()
                if 'timestamp' in response:
                    hour_trading.latest_time = response['timestamp']

                for key, value in detail.items():
                    if value is None:
                        continue
                    if isinstance(value, six.string_types):
                        value = get_string(value)
                    if key == 'tag':
                        if value == '盘前':
                            hour_trading.trading_session = TradingSession.PreMarket
                        elif value == '盘后':
                            hour_trading.trading_session = TradingSession.AfterHours
                    else:
                        tag = BRIEF_FIELD_MAPPINGS[key] if key in BRIEF_FIELD_MAPPINGS else key
                        if hasattr(hour_trading, tag):
                            setattr(hour_trading, tag, value)
                self.hour_trading = hour_trading
            if 'items' in data_json:
                timeline_items = []
                for item in data_json['items']:
                    item_values = {'pre_close': pre_close}
                    for key, value in item.items():
                        if value is None:
                            continue
                        if isinstance(value, six.string_types):
                            value = get_string(value)
                        tag = TIMELINE_FIELD_MAPPINGS[key] if key in TIMELINE_FIELD_MAPPINGS else key
                        item_values[tag] = value
                    timeline_items.append([item_values.get(tag) for tag in COLUMNS])

                self.timelines = pd.DataFrame(timeline_items, columns=COLUMNS)



# -*- coding: utf-8 -*-
"""
Created on 2018/11/1

@author: gaoan
"""
from enum import Enum


class RequestType(Enum):
    # 交易
    ORDER_NO = 1
    PREVIEW_ORDER = 2
    PLACE_ORDER = 3
    CANCEL_ORDER = 4
    MODIFY_ORDER = 5
    REQ_OPEN_ORDERS = 6
    REQ_ASSETS = 7
    REQ_POSITIONS = 8
    REQ_ACCOUNT = 9

    # 行情
    REQ_MARKET_STATE = 101
    REQ_ALL_SYMBOLS = 102
    REQ_ALL_SYMBOL_NAMES = 103
    REQ_BRIEF_INFO = 104
    REQ_STOCK_DETAIL = 105
    REQ_TIME_LINE = 106
    REQ_HOUR_TRADING_TIME_LINE = 107
    REQ_KLINE = 108
    REQ_TRADE_TICK = 109
    REQ_SUB_SYMBOLS = 110


class ResponseType(Enum):
    # 交易
    GET_ORDER_NO_END = 1
    PREVIEW_ORDER_END = 2
    PLACE_ORDER_END = 3
    CANCEL_ORDER_END = 4
    MODIFY_ORDER_END = 5
    GET_ASSET_END = 6
    GET_POSITION_END = 7
    GET_ACCOUNT_END = 8
    SUBSCRIBE_ORDER_STATUS = 9
    SUBSCRIBE_POSITION = 10
    SUBSCRIBE_ASSET = 11

    # 行情
    GET_MARKET_STATE_END = 101
    GET_ALL_SYMBOLS_END = 102
    GET_ALL_SYMBOL_NAMES_END = 103
    GET_BRIEF_INFO_END = 104
    GET_STOCK_DETAIL_END = 105
    GET_TIME_LINE_END = 106
    GET_HOUR_TRADING_TIME_LINE_END = 107
    GET_KLINE_END = 108
    GET_TRADING_TICK_END = 109
    GET_QUOTE_CHANGE_END = 110
    GET_SUB_SYMBOLS_END = 111
    GET_SUBSCRIBE_END = 112
    GET_CANCEL_SUBSCRIBE_END = 113

    ERROR_END = 200



from tigeropen.trade.trade_client import TradeClient
from tigeropen.push.push_client import PushClient
from tigeropen.quote.quote_client import QuoteClient
from tigeropen.common.consts import Language
from tigeropen.tiger_open_config import TigerOpenClientConfig
from tigeropen.common.util.signature_utils import read_private_key
from tigeropen.examples.tinyquant.context import global_context
from tigeropen.examples.tinyquant import setting


def get_client_config():
    """
    https://www.itiger.com/openapi/info 开发者信息获取
    :return:
    """
    client_config = TigerOpenClientConfig(sandbox_debug=setting.IS_SANDBOX)
    client_config.private_key = read_private_key(setting.PRIVATE_KEY)
    client_config.tiger_id = setting.TIGER_ID
    client_config.account = setting.ACCOUNT
    client_config.language = Language.en_US
    return client_config


client_config = get_client_config()

global_context.account = client_config.account

# ============= initialize push client & trade_client & quote_client ===================
protocol, host, port = client_config.socket_host_port
push_client = PushClient(host, port, use_ssl=(protocol == 'ssl'))
trade_client = TradeClient(client_config, logger=None)
quote_client = QuoteClient(client_config, logger=None)


# -*- coding: utf-8 -*-
# 
# @Date    : 2021-04-16
# @Author  : sukai
import json
from tigeropen.common.response import TigerResponse


class QuoteGrabPermissionResponse(TigerResponse):
    def __init__(self):
        super(QuoteGrabPermissionResponse, self).__init__()
        self.permissions = None
        self._is_success = None

    def parse_response_content(self, response_content):
        response = super(QuoteGrabPermissionResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']

        self.permissions = self.data



from functools import partial
from .client import trade_client
from .setting import IS_PAPER


class Portfolio:
    def __init__(self, context):
        self.context = context

    @property
    def cash(self):
        if IS_PAPER:
            assets = trade_client.get_assets()
            if assets:
                return assets[0].summary.cash
        else:
            return self.context.asset_manager.summary.cash

    @property
    def positions_value(self):
        if IS_PAPER:
            assets = trade_client.get_assets()
            if assets:
                return assets[0].summary.gross_position_value
        else:
            return self.context.asset_manager.summary.gross_position_value

    @property
    def portfolio_value(self):
        if IS_PAPER:
            assets = trade_client.get_assets()
            if assets:
                return assets[0].summary.net_liquidation
        return self.context.asset_manager.summary.net_liquidation

    @property
    def positions(self):
        if IS_PAPER:
            positions = trade_client.get_positions()
            return {position.contract.symbol: position for position in positions}
        else:
            return self.context.position_manager

    def __repr__(self):
        return '<Portfolio: net_liquidation: %s, cash: %s, positions_value: %s>' \
               % (self.portfolio_value, self.cash, self.positions_value)


class CompatibleStrategy:
    """与量化平台兼容的策略"""
    def __init__(self, push_client=None, trade_client=None, quote_client=None, context=None):
        self.push_client = push_client
        self.trade_client = trade_client
        self.quote_client = quote_client
        self.context = context

        self.context.portfolio = Portfolio(context)

        self.initialize = self.noop
        self.before_trading_start = self.noop
        self.handle_data = self.noop

        from . import strategy_event_trigger
        if hasattr(strategy_event_trigger, 'initialize'):
            self.initialize = partial(strategy_event_trigger.initialize, context=self.context)
        if hasattr(strategy_event_trigger, 'before_trading_start'):
            self.before_trading_start = partial(strategy_event_trigger.before_trading_start, context=self.context)
        if hasattr(strategy_event_trigger, 'handle_data'):
            self.handle_data = partial(strategy_event_trigger.handle_data, context=self.context)

    def noop(self, *args, **kwargs):
        pass


class TickStrategy:
    """行情驱动策略"""
    def __init__(self, push_client=None, trade_client=None, quote_client=None, context=None):
        self.push_client = push_client
        self.trade_client = trade_client
        self.quote_client = quote_client

        self.context = context
        self.context.push_client = push_client
        self.context.trade_client = trade_client
        self.context.quote_client = quote_client

        self.initialize = self.noop
        self.before_trading_start = self.noop
        self.on_ticker = self.noop
        self.dump = self.noop

        from . import strategy_quote_trigger
        if hasattr(strategy_quote_trigger, 'initialize'):
            self.initialize = partial(strategy_quote_trigger.initialize, context=self.context)
        if hasattr(strategy_quote_trigger, 'before_trading_start'):
            self.before_trading_start = partial(strategy_quote_trigger.before_trading_start, context=self.context)
        if hasattr(strategy_quote_trigger, 'on_ticker'):
            self.on_ticker = partial(strategy_quote_trigger.on_ticker, context=self.context)
        if hasattr(strategy_quote_trigger, 'dump'):
            self.dump = partial(strategy_quote_trigger.dump, context=self.context)

    def noop(self, *args, **kwargs):
        pass


# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""
import json

import six
import pandas as pd
from tigeropen.common.util.string_utils import get_string
from tigeropen.common.response import TigerResponse

# 盘前盘后
HOUR_TRADING_COLUMNS = [
    'hour_trading_tag',
    'hour_trading_latest_price',
    'hour_trading_pre_close',
    'hour_trading_latest_time',
    'hour_trading_volume',
    'hour_trading_timestamp',
]
# 下一交易时段信息
NEXT_MARKET_STATUS_COLUMNS = [
    # 市场状态(US:开盘、收盘、盘前交易、盘后交易||CN/HK：开盘、收盘、午间休市)
    'next_market_status_tag',
    # 开始时间
    'next_market_status_begin_time',
]
# 拆合股
STOCK_SPLIT_COLUMNS = [
    # 执行日期 yyyy-MM-dd格式
    'stock_split_execute_date',
    # 公司行动后的因子
    'stock_split_to_factor',
    # 公司行动前的因子
    'stock_split_for_factor'
]
# 股权信息
STOCK_RIGHT_COLUMNS = [
    # 原股权代码
    'stock_right_symbol',
    # 股权代码
    'stock_right_rights_symbol',
    # 开始交易日期, YYYY-MM-dd格式，可能为空字符串
    'stock_right_first_dealing_date',
    # 最后交易日期, YYYY-MM-dd格式，可能为空字符串
    'stock_right_last_dealing_date'
]
# 股票代码变更
SYMBOL_CHANGE_COLUMNS = [
    # 新的股票代码
    'symbol_change_new_symbol',
    # 执行日期，yyyy-MM-dd格式
    'symbol_change_execute_date'
]
# 股票公告
STOCK_NOTICE_COLUMNS = [
    # 公告标题
    'stock_notice_title',
    # 公告内容
    'stock_notice_content',
    # 公告类型
    'stock_notice_type'
]

COLUMNS = ['symbol', 'market', 'exchange', 'sec_type', 'name', 'shortable', 'latest_price', 'pre_close',
           'adj_pre_close', 'trading_status', 'market_status', 'timestamp', 'latest_time',
           'open', 'high', 'low', 'volume', 'amount', 'ask_price', 'ask_size', 'bid_price', 'bid_size', 'change',
           'amplitude', 'halted', 'delay', 'float_shares', 'shares', 'eps', 'etf', 'listing_date', 'adr_rate'
           ] + HOUR_TRADING_COLUMNS + NEXT_MARKET_STATUS_COLUMNS + STOCK_SPLIT_COLUMNS + STOCK_RIGHT_COLUMNS \
          + SYMBOL_CHANGE_COLUMNS + STOCK_NOTICE_COLUMNS

DETAIL_FIELD_MAPPINGS = {'secType': 'sec_type', 'latestPrice': 'latest_price', 'preClose': 'pre_close',
                         'floatShares': 'float_shares', 'marketStatus': 'market_status', 'latestTime': 'latest_time',
                         'askPrice': 'ask_price', 'askSize': 'ask_size', 'bidPrice': 'bid_price', 'bidSize': 'bid_size',
                         'tradingStatus': 'trading_status', 'adjPreClose': 'adj_pre_close', 'adrRate': 'adr_rate',
                         'listingDate': 'listing_date', 'beginTime': 'begin_time',
                         'nextMarketStatus': 'next_market_status', 'hourTrading': 'hour_trading',
                         'stockSplit': 'stock_split', 'stockRight': 'stock_right', 'symbolChange': 'symbol_change',
                         'executeDate': 'execute_date', 'newSymbol': 'new_symbol', 'forFactor': 'for_factor',
                         'toFactor': 'to_factor', 'rightsSymbol': 'rights_symbol', 'stockNotice': 'stock_notice',
                         'firstDealingDate': 'first_dealing_date', 'lastDealingDate': 'last_dealing_date'
                         }

SUB_FIELDS = {
    # 下一交易时段信息
    'next_market_status',
    # 盘前盘后信息
    'hour_trading',
    # 拆合股
    'stock_split',
    # 股权信息
    'stock_right'
    # 股票代码变更
    'symbol_change',
    # 公告
    'stock_notice'
}


class StockDetailsResponse(TigerResponse):
    def __init__(self):
        super(StockDetailsResponse, self).__init__()
        self.details = None
        self._is_success = None

    def parse_response_content(self, response_content):
        response = super(StockDetailsResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']
        if not self.data:
            return
        detail_data = []
        data_json = json.loads(self.data)
        for item in data_json.get('items', {}):
            item_values = dict()
            for key, value in item.items():
                if value is None:
                    continue
                if isinstance(value, six.string_types):
                    value = get_string(value)
                tag = self._key_to_tag(key)
                if tag in SUB_FIELDS:
                    for sub_k, sub_v in value.items():
                        sub_tag = self._key_to_tag(sub_k)
                        item_values[self._join_tag(tag, sub_tag)] = sub_v
                else:
                    item_values[tag] = value

            detail_data.append([item_values.get(tag) for tag in COLUMNS])

        self.details = pd.DataFrame(detail_data, columns=COLUMNS)

    @classmethod
    def _key_to_tag(cls, k):
        return DETAIL_FIELD_MAPPINGS[k] if k in DETAIL_FIELD_MAPPINGS else k

    @classmethod
    def _join_tag(cls, tag, sub_tag):
        return '_'.join((tag, sub_tag))



# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""
import json
import six
from tigeropen.common.response import TigerResponse
from tigeropen.common.util.string_utils import get_string
from tigeropen.trade.domain.contract import Contract
from tigeropen.trade.domain.position import Position
from tigeropen.trade.response import CONTRACT_FIELDS

POSITION_FIELD_MAPPINGS = {'averageCost': 'average_cost', 'position': 'quantity', 'latestPrice': 'market_price',
                           'marketValue': 'market_value', 'orderType': 'order_type', 'realizedPnl': 'realized_pnl',
                           'unrealizedPnl': 'unrealized_pnl', 'secType': 'sec_type', 'localSymbol': 'local_symbol',
                           'originSymbol': 'origin_symbol', 'contractId': 'contract_id', 'identifier': 'identifier'}


class PositionsResponse(TigerResponse):
    def __init__(self):
        super(PositionsResponse, self).__init__()
        self.positions = []
        self._is_success = None
    
    def parse_response_content(self, response_content):
        response = super(PositionsResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']
        
        if self.data:
            data_json = json.loads(self.data)
            if 'items' in data_json:
                for item in data_json['items']:
                    contract_fields = {}
                    position_fields = {}
                    for key, value in item.items():
                        if value is None:
                            continue
                        if isinstance(value, six.string_types):
                            value = get_string(value)
                        tag = POSITION_FIELD_MAPPINGS[key] if key in POSITION_FIELD_MAPPINGS else key
                        if tag in CONTRACT_FIELDS:
                            contract_fields[tag] = value
                        else:
                            position_fields[tag] = value
                    
                    contract_id = contract_fields.get('contract_id')
                    symbol = contract_fields.get('symbol')
                    currency = contract_fields.get('currency')
                    sec_type = contract_fields.get('sec_type')
                    exchange = contract_fields.get('exchange')
                    origin_symbol = contract_fields.get('origin_symbol')
                    local_symbol = contract_fields.get('local_symbol')
                    expiry = contract_fields.get('expiry')
                    strike = contract_fields.get('strike')
                    put_call = contract_fields.get('right')
                    multiplier = contract_fields.get('multiplier')
                    identifier = contract_fields.get('identifier')
                    contract = Contract(symbol, currency, contract_id=contract_id, sec_type=sec_type,
                                        exchange=exchange, origin_symbol=origin_symbol, local_symbol=local_symbol,
                                        expiry=expiry, strike=strike, put_call=put_call, multiplier=multiplier,
                                        identifier=identifier)
                    account = position_fields.get('account')
                    quantity = position_fields.get('quantity')
                    average_cost = position_fields.get('average_cost')
                    market_price = position_fields.get('market_price')
                    market_value = position_fields.get('market_value')
                    realized_pnl = position_fields.get('realized_pnl')
                    unrealized_pnl = position_fields.get('unrealized_pnl')
                    
                    position = Position(account, contract, quantity, average_cost=average_cost,
                                        market_price=market_price, market_value=market_value,
                                        realized_pnl=realized_pnl, unrealized_pnl=unrealized_pnl)
                    self.positions.append(position)



# -*- coding: utf-8 -*-
"""
Created on 2018/9/20

@author: gaoan
"""
from tigeropen.common.consts import Language
from tigeropen.common.util.signature_utils import read_private_key

# 老虎证券开放平台网关地址
SERVER_URL = 'https://openapi.itiger.com/gateway'
# 老虎证券开放平台 socket 连接域名端口
SOCKET_HOST_PORT = ('ssl', 'openapi.itiger.com', 8883)
# 老虎证券开放平台公钥
TIGER_PUBLIC_KEY = 'MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDNF3G8SoEcCZh2rshUbayDgLLrj6rKgzNMxDL2HS' \
                   'nKcB0+GPOsndqSv+a4IBu9+I3fyBp5hkyMMG2+AXugd9pMpy6VxJxlNjhX1MYbNTZJUT4nudki4uh+LM' \
                   'OkIBHOceGNXjgB+cXqmlUnjlqha/HgboeHSnSgpM3dKSJQlIOsDwIDAQAB'
# 请求签名类型
SIGN_TYPE = 'RSA'
# 请求字符集
CHARSET = 'UTF-8'
# 语言
LANGUAGE = Language.zh_CN
# 请求超时时间, 单位秒, 默认15s
TIMEOUT = 15

# sandbox 环境配置
SANDBOX_SERVER_URL = 'https://openapi-sandbox.itiger.com/gateway'
SANDBOX_SOCKET_HOST_PORT = ('ssl', 'openapi-sandbox.itiger.com', 8885)
SANDBOX_TIGER_PUBLIC_KEY = 'MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCbm21i11hgAENGd3/f280PSe4g9YGkS3TEXBY' \
                           'MidihTvHHf+tJ0PYD0o3PruI0hl3qhEjHTAxb75T5YD3SGK4IBhHn/Rk6mhqlGgI+bBrBVYaXixm' \
                           'HfRo75RpUUuWACyeqQkZckgR0McxuW9xRMIa2cXZOoL1E4SL4lXKGhKoWbwIDAQAB'


class TigerOpenClientConfig(object):
    def __init__(self, sandbox_debug=False):
        # 开发者应用id
        self._tiger_id = ''
        # 授权账户
        self._account = ''
        # 综合账户
        self._standard_account = ''
        # 模拟账户
        self._paper_account = ''
        # 开发者应用私钥
        self._private_key = ''
        # 请求签名类型，推荐RSA2
        self._sign_type = SIGN_TYPE
        # 机构交易员专有密钥
        self._secret_key = ''

        # 老虎证券开放平台网关地址
        self._server_url = SERVER_URL
        self._socket_host_port = SOCKET_HOST_PORT
        # 老虎证券开放平台公钥
        self._tiger_public_key = TIGER_PUBLIC_KEY

        if sandbox_debug:
            self._server_url = SANDBOX_SERVER_URL
            self._socket_host_port = SANDBOX_SOCKET_HOST_PORT
            self._tiger_public_key = SANDBOX_TIGER_PUBLIC_KEY
        # 请求字符集，默认utf-8
        self._charset = CHARSET
        # 语言
        self._language = LANGUAGE
        
        # 以下为可选参数
        # 请求读取超时，单位秒，默认15s
        self._timeout = TIMEOUT
    
    @property
    def tiger_id(self):
        return self._tiger_id
    
    @tiger_id.setter
    def tiger_id(self, value):
        self._tiger_id = value
    
    @property
    def account(self):
        return self._account
    
    @account.setter
    def account(self, value):
        self._account = value

    @property
    def standard_account(self):
        return self._standard_account

    @standard_account.setter
    def standard_account(self, value):
        self._standard_account = value

    @property
    def paper_account(self):
        return self._paper_account

    @paper_account.setter
    def paper_account(self, value):
        self._paper_account = value
    
    @property
    def sign_type(self):
        return self._sign_type
    
    @sign_type.setter
    def sign_type(self, value):
        self._sign_type = value
    
    @property
    def private_key(self):
        return self._private_key
    
    @private_key.setter
    def private_key(self, value):
        self._private_key = value
    
    @property
    def tiger_public_key(self):
        return self._tiger_public_key
    
    @tiger_public_key.setter
    def tiger_public_key(self, value):
        self._tiger_public_key = value
    
    @property
    def server_url(self):
        return self._server_url
    
    @server_url.setter
    def server_url(self, value):
        self._server_url = value
    
    @property
    def socket_host_port(self):
        return self._socket_host_port
    
    @socket_host_port.setter
    def socket_host_port(self, value):
        self._socket_host_port = value
    
    @property
    def charset(self):
        return self._charset
    
    @charset.setter
    def charset(self, value):
        self._charset = value
    
    @property
    def language(self):
        return self._language
    
    @language.setter
    def language(self, value):
        self._language = value
    
    @property
    def timeout(self):
        return self._timeout
    
    @timeout.setter
    def timeout(self, value):
        self._timeout = value

    @property
    def secret_key(self):
        return self._secret_key

    @secret_key.setter
    def secret_key(self, value):
        self._secret_key = value


def get_client_config(private_key_path, tiger_id, account, standard_account=None, paper_account=None,
                      sandbox_debug=False, sign_type=None, timeout=None, language=None, charset=None,
                      server_url=None, socket_host_port=None, secret_key=None):
    """
    生成客户端配置
    :param private_key_path: 私钥文件路径, 如 '/Users/tiger/.ssh/rsa_private_key.pem'
    :param tiger_id: 开发者应用 id
    :param account: 授权账户 (必填. 作为发送请求时的默认账户. 不论是环球账户, 综合账户或是模拟账户, 都使用此参数)
    :param standard_account: 多账户时可将综合账户填在此处, 一般可忽略
    :param paper_account: 多账户时可将模拟账户填在此处, 一般可忽略
    :param sandbox_debug: 是否请求 sandbox 环境
    :param sign_type: 签名类型
    :param timeout: 请求超时时间, 单位秒
    :param language: 语言, 取值为 tigeropen.common.consts.Language 中的枚举类型
    :param charset: 字符集编码
    :param server_url: 网关地址
    :param socket_host_port: 推送长连接的域名端口, 值为协议, 域名, 端口构成的三元组
    :param secret_key: 机构交易员专有密钥 (个人开发者无需指定)
    :return:
    """
    config = TigerOpenClientConfig(sandbox_debug=sandbox_debug)
    config.private_key = read_private_key(private_key_path)
    config.tiger_id = tiger_id
    config.account = account
    if standard_account:
        config.standard_account = standard_account
    if paper_account:
        config.paper_account = paper_account
    if sign_type:
        config.sign_type = sign_type
    if timeout:
        config.timeout = timeout
    if language:
        config.language = language
    if charset:
        config.charset = charset
    if server_url:
        config.server_url = server_url
    if socket_host_port:
        config.socket_host_port = socket_host_port
    if secret_key:
        config.secret_key = secret_key
    return config




# -*- coding: utf-8 -*-
"""
Created on 2018/9/20

@author: gaoan
"""


class MarketParams(object):
    def __init__(self):
        self._market = None  # 市场
        self._sec_type = None  # 交易品种
        self._lang = None  # 语言

    @property
    def market(self):
        return self._market

    @market.setter
    def market(self, value):
        self._market = value

    @property
    def sec_type(self):
        return self._sec_type

    @sec_type.setter
    def sec_type(self, value):
        self._sec_type = value

    @property
    def lang(self):
        return self._lang

    @lang.setter
    def lang(self, value):
        self._lang = value

    def to_openapi_dict(self):
        params = dict()
        if self.market:
            params['market'] = self.market

        if self.sec_type:
            params['sec_type'] = self.sec_type

        if self.lang:
            params['lang'] = self.lang

        return params


class SingleQuoteParams(MarketParams):
    def __init__(self):
        super(SingleQuoteParams, self).__init__()
        self._symbol = None
        self._put_call = None  # for option
        self._expiry = None  # for option and future
        self._strike = None  # for option and future
        self._include_hour_trading = None
        self._include_ask_bid = None
        self._right = None
        self._period = None
        self._begin_time = None
        self._end_time = None
        self._limit = None
        self._begin_index = None
        self._end_index = None

    @property
    def symbol(self):
        return self._symbol

    @symbol.setter
    def symbol(self, value):
        self._symbol = value

    @property
    def put_call(self):
        return self._put_call

    @put_call.setter
    def put_call(self, value):
        self._put_call = value

    @property
    def expiry(self):
        return self._expiry

    @expiry.setter
    def expiry(self, value):
        self._expiry = value

    @property
    def strike(self):
        return self._strike

    @strike.setter
    def strike(self, value):
        self._strike = value

    @property
    def include_hour_trading(self):
        return self._include_hour_trading

    @include_hour_trading.setter
    def include_hour_trading(self, value):
        self._include_hour_trading = value

    @property
    def include_ask_bid(self):
        return self._include_ask_bid

    @include_ask_bid.setter
    def include_ask_bid(self, value):
        self._include_ask_bid = value

    @property
    def right(self):
        return self._right

    @right.setter
    def right(self, value):
        self._right = value

    @property
    def period(self):
        return self._period

    @period.setter
    def period(self, value):
        self._period = value

    @property
    def begin_time(self):
        return self._begin_time

    @begin_time.setter
    def begin_time(self, value):
        self._begin_time = value

    @property
    def end_time(self):
        return self._end_time

    @end_time.setter
    def end_time(self, value):
        self._end_time = value

    @property
    def begin_index(self):
        return self._begin_index

    @begin_index.setter
    def begin_index(self, value):
        self._begin_index = value

    @property
    def end_index(self):
        return self._end_index

    @end_index.setter
    def end_index(self, value):
        self._end_index = value

    @property
    def limit(self):
        return self._limit

    @limit.setter
    def limit(self, value):
        self._limit = value

    def to_openapi_dict(self):
        params = super(SingleQuoteParams, self).to_openapi_dict()

        if self.symbol:
            params['symbol'] = self.symbol

        if self.put_call:
            params['right'] = self.put_call

        if self.expiry:
            params['expiry'] = self.expiry

        if self.strike:
            params['strike'] = self.strike

        if self.include_hour_trading is not None:
            params['include_hour_trading'] = self.include_hour_trading

        if self.include_ask_bid is not None:
            params['include_ask_bid'] = self.include_ask_bid

        if self.right:
            params['right'] = self.right

        if self.period:
            params['period'] = self.period

        if self.begin_time:
            params['begin_time'] = self.begin_time

        if self.end_time:
            params['end_time'] = self.end_time

        if self.begin_index is not None:
            params['begin_index'] = self.begin_index

        if self.end_index is not None:
            params['end_index'] = self.end_index

        if self.limit:
            params['limit'] = self.limit

        return params


class MultipleQuoteParams(MarketParams):
    def __init__(self):
        super(MultipleQuoteParams, self).__init__()
        self._symbols = None
        self._include_hour_trading = None
        self._include_ask_bid = None
        self._right = None
        self._period = None
        self._begin_time = None
        self._end_time = None
        self._limit = None
        self._begin_index = None
        self._end_index = None

    @property
    def symbols(self):
        return self._symbols

    @symbols.setter
    def symbols(self, value):
        self._symbols = value

    @property
    def include_hour_trading(self):
        return self._include_hour_trading

    @include_hour_trading.setter
    def include_hour_trading(self, value):
        self._include_hour_trading = value

    @property
    def include_ask_bid(self):
        return self._include_ask_bid

    @include_ask_bid.setter
    def include_ask_bid(self, value):
        self._include_ask_bid = value

    @property
    def right(self):
        return self._right

    @property
    def period(self):
        return self._period

    @period.setter
    def period(self, value):
        self._period = value

    @right.setter
    def right(self, value):
        self._right = value

    @property
    def begin_time(self):
        return self._begin_time

    @begin_time.setter
    def begin_time(self, value):
        self._begin_time = value

    @property
    def end_time(self):
        return self._end_time

    @end_time.setter
    def end_time(self, value):
        self._end_time = value

    @property
    def limit(self):
        return self._limit

    @limit.setter
    def limit(self, value):
        self._limit = value

    @property
    def begin_index(self):
        return self._begin_index

    @begin_index.setter
    def begin_index(self, value):
        self._begin_index = value

    @property
    def end_index(self):
        return self._end_index

    @end_index.setter
    def end_index(self, value):
        self._end_index = value

    def to_openapi_dict(self):
        params = super(MultipleQuoteParams, self).to_openapi_dict()

        if self.symbols:
            params['symbols'] = self.symbols

        if self.include_hour_trading is not None:
            params['include_hour_trading'] = self.include_hour_trading

        if self.include_ask_bid is not None:
            params['include_ask_bid'] = self.include_ask_bid

        if self.right:
            params['right'] = self.right

        if self.period:
            params['period'] = self.period

        if self.begin_time:
            params['begin_time'] = self.begin_time

        if self.end_time:
            params['end_time'] = self.end_time

        if self.limit:
            params['limit'] = self.limit

        if self.begin_index:
            params['begin_index'] = self.begin_index

        if self.end_index:
            params['end_index'] = self.end_index

        return params


class SingleContractParams(object):
    def __init__(self):
        self._symbol = None
        self._put_call = None  # for option
        self._expiry = None  # for option and future
        self._strike = None  # for option and future

    @property
    def symbol(self):
        return self._symbol

    @symbol.setter
    def symbol(self, value):
        self._symbol = value

    @property
    def put_call(self):
        return self._put_call

    @put_call.setter
    def put_call(self, value):
        self._put_call = value

    @property
    def expiry(self):
        return self._expiry

    @expiry.setter
    def expiry(self, value):
        self._expiry = value

    @property
    def strike(self):
        return self._strike

    @strike.setter
    def strike(self, value):
        self._strike = value

    def to_openapi_dict(self):
        params = dict()

        if self.symbol:
            params['symbol'] = self.symbol

        if self.put_call:
            params['right'] = self.put_call

        if self.expiry:
            params['expiry'] = self.expiry

        if self.strike:
            params['strike'] = self.strike

        return params


class SingleOptionQuoteParams(SingleContractParams):
    def __init__(self):
        super(SingleOptionQuoteParams, self).__init__()
        self._period = None
        self._begin_time = None
        self._end_time = None
        self._limit = None

    @property
    def period(self):
        return self._period

    @period.setter
    def period(self, value):
        self._period = value

    @property
    def begin_time(self):
        return self._begin_time

    @begin_time.setter
    def begin_time(self, value):
        self._begin_time = value

    @property
    def end_time(self):
        return self._end_time

    @end_time.setter
    def end_time(self, value):
        self._end_time = value

    @property
    def limit(self):
        return self._limit

    @limit.setter
    def limit(self, value):
        self._limit = value

    def to_openapi_dict(self):
        params = super(SingleOptionQuoteParams, self).to_openapi_dict()

        if self.period:
            params['period'] = self.period

        if self.begin_time:
            params['begin_time'] = self.begin_time

        if self.end_time:
            params['end_time'] = self.end_time

        if self.limit:
            params['limit'] = self.limit

        return params


class MultipleContractParams(object):
    def __init__(self):
        super(MultipleContractParams, self).__init__()
        self._contracts = None  # list of SingleQuoteParams

    @property
    def contracts(self):
        return self._contracts

    @contracts.setter
    def contracts(self, value):
        self._contracts = value

    def to_openapi_dict(self):
        params = list()

        if self.contracts:
            for contract in self.contracts:
                params.append(contract.to_openapi_dict())

        return params


class FutureExchangeParams(object):
    def __init__(self):
        self._exchange_code = None  # 交易所
        self._lang = None  # 语言

    @property
    def exchange_code(self):
        return self._exchange_code

    @exchange_code.setter
    def exchange_code(self, value):
        self._exchange_code = value

    @property
    def lang(self):
        return self._lang

    @lang.setter
    def lang(self, value):
        self._lang = value

    def to_openapi_dict(self):
        params = dict()
        if self.exchange_code:
            params['exchange_code'] = self.exchange_code

        if self.lang:
            params['lang'] = self.lang

        return params


class FutureTypeParams(object):
    def __init__(self):
        self._type = None  # 期货品种
        self._lang = None  # 语言

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, value):
        self._type = value

    @property
    def lang(self):
        return self._lang

    @lang.setter
    def lang(self, value):
        self._lang = value

    def to_openapi_dict(self):
        params = dict()
        if self.type:
            params['type'] = self.type

        if self.lang:
            params['lang'] = self.lang

        return params


class FutureTradingTimeParams(object):
    def __init__(self):
        self._contract_code = None
        self._trading_date = None

    @property
    def contract_code(self):
        return self._contract_code

    @contract_code.setter
    def contract_code(self, value):
        self._contract_code = value

    @property
    def trading_date(self):
        return self._trading_date

    @trading_date.setter
    def trading_date(self, value):
        self._trading_date = value

    def to_openapi_dict(self):
        params = dict()
        if self.contract_code:
            params['contract_code'] = self.contract_code

        if self.trading_date:
            params['trading_date'] = self.trading_date

        return params


class FutureQuoteParams(MarketParams):
    def __init__(self):
        super(FutureQuoteParams, self).__init__()
        self._contract_codes = None
        self._period = None
        self._begin_time = None
        self._end_time = None
        self._limit = None
        self._begin_index = None
        self._end_index = None

    @property
    def contract_codes(self):
        return self._contract_codes

    @contract_codes.setter
    def contract_codes(self, value):
        self._contract_codes = value

    @property
    def period(self):
        return self._period

    @period.setter
    def period(self, value):
        self._period = value

    @property
    def begin_time(self):
        return self._begin_time

    @begin_time.setter
    def begin_time(self, value):
        self._begin_time = value

    @property
    def end_time(self):
        return self._end_time

    @end_time.setter
    def end_time(self, value):
        self._end_time = value

    @property
    def begin_index(self):
        return self._begin_index

    @begin_index.setter
    def begin_index(self, value):
        self._begin_index = value

    @property
    def end_index(self):
        return self._end_index

    @end_index.setter
    def end_index(self, value):
        self._end_index = value

    @property
    def limit(self):
        return self._limit

    @limit.setter
    def limit(self, value):
        self._limit = value

    def to_openapi_dict(self):
        params = super(FutureQuoteParams, self).to_openapi_dict()

        if self.contract_codes:
            params['contract_codes'] = self.contract_codes

        if self.period:
            params['period'] = self.period

        if self.begin_time:
            params['begin_time'] = self.begin_time

        if self.end_time:
            params['end_time'] = self.end_time

        if self.begin_index is not None:
            params['begin_index'] = self.begin_index

        if self.end_index is not None:
            params['end_index'] = self.end_index

        if self.limit:
            params['limit'] = self.limit

        return params


class DepthQuoteParams(object):
    def __init__(self):
        self._symbols = None
        self._market = None

    @property
    def symbols(self):
        return self._symbols

    @symbols.setter
    def symbols(self, value):
        self._symbols = value

    @property
    def market(self):
        return self._market

    @market.setter
    def market(self, value):
        self._market = value

    def to_openapi_dict(self):
        params = dict()
        if self.symbols:
            params['symbols'] = self.symbols
        if self.market:
            params['market'] = self.market
        return params



# -*- coding: utf-8 -*-

from tigeropen.common.util.string_utils import get_string
from tigeropen.common.response import TigerResponse


class IndustryListResponse(TigerResponse):
    def __init__(self):
        super(IndustryListResponse, self).__init__()
        self.industry_list = list()
        self._is_success = None

    def parse_response_content(self, response_content):
        response = super(IndustryListResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']
        if self.data:
            for ind in self.data:
                industry = dict(industry_level=ind.get('industryLevel'), id=ind.get('id'),
                                name_cn=get_string(ind.get('nameCN')),
                                name_en=get_string(ind.get('nameEN')))
                self.industry_list.append(industry)


class IndustryStocksResponse(TigerResponse):
    def __init__(self):
        super(IndustryStocksResponse, self).__init__()
        self.industry_stocks = list()
        self._is_success = None

    def parse_response_content(self, response_content):
        response = super(IndustryStocksResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']
        if self.data:
            for item in self.data:
                industry_list = list()
                industries = item.get('industryDetailDTOList', [])
                for ind in industries:
                    industry_list.append(dict(industry_level=ind.get('industryLevel'), id=ind.get('id'),
                                              name_cn=get_string(ind.get('nameCN')),
                                              name_en=get_string(ind.get('nameEN'))))
                company = dict(symbol=item.get('symbol'), company_name=item.get('companyName'),
                               market=item.get('market'), industry_list=industry_list)
                self.industry_stocks.append(company)


class StockIndustryResponse(TigerResponse):
    def __init__(self):
        super(StockIndustryResponse, self).__init__()
        self.stock_industry = list()
        self._is_success = None

    def parse_response_content(self, response_content):
        response = super(StockIndustryResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']
        if self.data:
            for item in self.data:
                industry = dict(industry_level=item.get('industryLevel'), id=item.get('id'),
                                name_cn=get_string(item.get('nameCN')),
                                name_en=get_string(item.get('nameEN')))
                self.stock_industry.append(industry)



# -*- coding: utf-8 -*-

QUOTE = 'quote'
QUOTE_DEPTH = 'quotedepth'
QUOTE_FUTURE = 'future'
QUOTE_OPTION = 'option'

TRADE_ASSET = 'trade/asset'
TRADE_POSITION = 'trade/position'
TRADE_ORDER = 'trade/order'



# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""
import json
import six
import pandas as pd

from tigeropen.common.util.string_utils import get_string
from tigeropen.common.response import TigerResponse

COLUMNS = ['symbol', 'time', 'open', 'high', 'low', 'close', 'volume']
BAR_FIELD_MAPPINGS = {'avgPrice': 'avg_price'}


class QuoteBarResponse(TigerResponse):
    def __init__(self):
        super(QuoteBarResponse, self).__init__()
        self.bars = None
        self._is_success = None

    def parse_response_content(self, response_content):
        response = super(QuoteBarResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']

        if self.data and isinstance(self.data, list):
            bar_items = []
            for symbol_item in self.data:
                symbol = symbol_item.get('symbol')
                if 'items' in symbol_item:
                    for item in symbol_item['items']:
                        item_values = {'symbol': symbol}
                        for key, value in item.items():
                            if value is None:
                                continue
                            if isinstance(value, six.string_types):
                                value = get_string(value)
                            tag = BAR_FIELD_MAPPINGS[key] if key in BAR_FIELD_MAPPINGS else key
                            item_values[tag] = value
                        bar_items.append([item_values.get(tag) for tag in COLUMNS])

            self.bars = pd.DataFrame(bar_items, columns=COLUMNS)



# -*- coding: utf-8 -*-

import pandas as pd
import six
from tigeropen.common.util.string_utils import get_string
from tigeropen.common.response import TigerResponse

COLUMNS = ['symbol', 'currency', 'field', 'value', 'period_end_date', 'filing_date']
REPORT_FIELD_MAPPINGS = {'periodEndDate': 'period_end_date', 'filingDate': 'filing_date'}


class FinancialReportResponse(TigerResponse):
    def __init__(self):
        super(FinancialReportResponse, self).__init__()
        self.financial_report = None
        self._is_success = None

    def parse_response_content(self, response_content):
        response = super(FinancialReportResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']

        if self.data and isinstance(self.data, list):
            items = list()
            for item in self.data:
                item_values = dict()
                for key, value in item.items():
                    if isinstance(value, six.string_types):
                        value = get_string(value)
                    item_values[key] = value
                items.append(item_values)
            self.financial_report = pd.DataFrame(items).rename(columns=REPORT_FIELD_MAPPINGS)[COLUMNS]



# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""
import six
import pandas as pd

from tigeropen.common.util.string_utils import get_string
from tigeropen.common.util.common_utils import eastern
from tigeropen.common.util.contract_utils import get_option_identifier
from tigeropen.common.response import TigerResponse

COLUMNS = ['identifier', 'symbol', 'expiry', 'put_call', 'strike', 'time', 'open', 'high', 'low', 'close', 'volume',
           'open_interest']
BAR_FIELD_MAPPINGS = {'avgPrice': 'avg_price', 'openInterest': 'open_interest', 'right': 'put_call'}


class OptionQuoteBarResponse(TigerResponse):
    def __init__(self):
        super(OptionQuoteBarResponse, self).__init__()
        self.bars = []
        self._is_success = None

    def parse_response_content(self, response_content):
        response = super(OptionQuoteBarResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']

        if self.data and isinstance(self.data, list):
            bar_items = []
            for symbol_item in self.data:
                identifier = symbol_item.get('identifier')
                underlying_symbol = symbol_item.get('symbol')
                expiry = symbol_item.get('expiry')
                strike = float(symbol_item.get('strike'))
                put_call = symbol_item.get('right')
                if put_call:
                    put_call = put_call.upper()

                if not identifier:
                    expiration = pd.Timestamp(expiry, unit='ms', tzinfo=eastern).date().strftime("%Y%m%d")
                    identifier = get_option_identifier(underlying_symbol, expiration, put_call, strike)

                if 'items' in symbol_item:
                    for item in symbol_item['items']:
                        item_values = {'identifier': identifier, 'symbol': underlying_symbol, 'expiry': expiry,
                                       'put_call': put_call, 'strike': strike}
                        for key, value in item.items():
                            if value is None:
                                continue
                            if isinstance(value, six.string_types):
                                value = get_string(value)
                            tag = BAR_FIELD_MAPPINGS[key] if key in BAR_FIELD_MAPPINGS else key
                            item_values[tag] = value
                        bar_items.append([item_values.get(tag) for tag in COLUMNS])

            self.bars = pd.DataFrame(bar_items, columns=COLUMNS)



# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""
import json
from tigeropen.common.response import TigerResponse


class OrderIdResponse(TigerResponse):
    def __init__(self):
        super(OrderIdResponse, self).__init__()
        self.order_id = None
        self.id = None
        self._is_success = None
    
    def parse_response_content(self, response_content):
        response = super(OrderIdResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']
        
        if self.data:
            data_json = json.loads(self.data)
            if 'code' in data_json and data_json['code'] != '0':
                self.code = int(data_json['code'])
                if 'message' in data_json:
                    self.message = data_json['message']
            
            if 'orderId' in data_json:
                self.order_id = data_json['orderId']

            if 'id' in data_json:
                self.id = data_json['id']



# -*- coding: utf-8 -*-
"""
Created on 2018/11/1

@author: gaoan
"""
import re

from tigeropen.common.consts import SecurityType
from tigeropen.trade.domain.contract import Contract


def stock_contract(symbol, currency, local_symbol=None, exchange=None, contract_id=None):
    return Contract(symbol, currency, sec_type='STK', local_symbol=local_symbol, exchange=exchange,
                    contract_id=contract_id)


def option_contract_by_symbol(symbol, expiry, strike, put_call, currency, multiplier=100, local_symbol=None,
                              contract_id=None):
    return Contract(symbol, currency, sec_type='OPT', expiry=expiry, strike=strike, put_call=put_call,
                    multiplier=multiplier, local_symbol=local_symbol, contract_id=contract_id)


def option_contract(identifier, multiplier=100, currency='USD'):
    symbol, expiry, put_call, strike = extract_option_info(identifier)
    if expiry and '-' in expiry:
        expiry = expiry.replace('-', '')
    return Contract(symbol, currency, sec_type='OPT', expiry=expiry, strike=strike, put_call=put_call,
                    multiplier=multiplier)


def future_contract(symbol, currency, expiry=None, exchange=None, contract_month=None, multiplier=None,
                    local_symbol=None):
    return Contract(symbol, currency, sec_type='FUT', expiry=expiry, exchange=exchange, contract_month=contract_month,
                    multiplier=multiplier, local_symbol=local_symbol)


def future_option_contract(symbol, currency, expiry, strike, put_call, multiplier=None, local_symbol=None,
                           contract_id=None):
    return Contract(symbol, currency, sec_type='FOP', expiry=expiry, strike=strike, put_call=put_call,
                    multiplier=multiplier, local_symbol=local_symbol, contract_id=contract_id)


def cash_contract(symbol, currency, local_symbol=None):
    return Contract(symbol, currency, sec_type='CASH', local_symbol=local_symbol)


def war_contract_by_symbol(symbol, expiry, strike, put_call, local_symbol, multiplier=100, currency='HKD',
                           contract_id=None):
    """港股窝轮"""
    return Contract(symbol, currency=currency, sec_type=SecurityType.WAR.value, expiry=expiry, strike=strike,
                    put_call=put_call, local_symbol=local_symbol, multiplier=multiplier, contract_id=contract_id)


def iopt_contract_by_symbol(symbol, expiry, strike, put_call, local_symbol, multiplier=100, currency='HKD',
                            contract_id=None):
    """港股牛熊证"""
    return Contract(symbol, currency=currency, sec_type=SecurityType.IOPT.value, expiry=expiry, strike=strike,
                    put_call=put_call, local_symbol=local_symbol, multiplier=multiplier, contract_id=contract_id)


def extract_option_info(identifier):
    """
    从期权中提取 symbol, expiry 等信息
    :param identifier:
    :return:
    """
    if identifier:
        tokens = re.findall(r'(\w+)\s*(\d{6})([CP])(\d+)', identifier, re.M)
        if len(tokens) == 1:
            underlying_symbol, expiry, put_call, strike = tokens[0]
            expiry = '20' + expiry
            if len(expiry) == 8:
                expiry = expiry[:4] + '-' + expiry[4:6] + '-' + expiry[6:]
            strike = float(strike) / 1000
            put_call = 'CALL' if put_call == 'C' else 'PUT'
            return underlying_symbol, expiry, put_call, strike
        else:
            return None, None, None, None


def get_option_identifier(underlying_symbol, expiry, put_call, strike):
    """
    获取期权合约的唯一标识
    :param underlying_symbol: like AAPL
    :param expiry: like 20100101
    :param put_call: like C or CALL or P or PUT
    :param strike: like 5.0
    :return:
    """
    direction = 'C' if put_call == 'C' or put_call == 'CALL' else 'P'
    return underlying_symbol.ljust(6, ' ') + expiry[2:] + direction + str(int(strike * 1000)).zfill(8)



# -*- coding: utf-8 -*-
"""
Created on 2018/9/20

@author: gaoan
"""


class Position(object):
    def __init__(self, account, contract, quantity=0, average_cost=None, market_price=None, market_value=None,
                 realized_pnl=None, unrealized_pnl=None):
        """
        - account: 对应的账户ID
        - contract: 合约对象
        - quantity: 合约数量
        - average_cost: 含佣金的平均成本
        - market_price: 市价
        - market_value: 市值
        - realized_pnl: 已实现盈亏
        - unrealized_pnl: 未实现盈亏
        """
        self.account = account
        self.contract = contract
        self.quantity = quantity
        self.average_cost = average_cost
        self.market_price = market_price
        self.market_value = market_value
        self.realized_pnl = realized_pnl
        self.unrealized_pnl = unrealized_pnl
    
    def __repr__(self):
        template = "contract: {contract}, quantity: {quantity}, average_cost: {average_cost}, " \
                   "market_price: {market_price}"
        return template.format(
            contract=self.contract,
            quantity=self.quantity,
            average_cost=self.average_cost,
            market_price=self.market_price
        )
    
    def __str__(self):
        return self.__repr__()
    
    def to_dict(self):
        """
        Creates a dictionary representing the state of this position.
        Returns a dict object of the form:
        """
        return {
            'contract': self.contract,
            'quantity': self.quantity,
            'average_cost': self.average_cost,
            'market_price': self.market_price
        }



# -*- coding: utf-8 -*-
"""
Created on 2018/9/20

@author: gaoan
"""
import logging
from collections import defaultdict


class Account(object):
    """
    The account object tracks information about the trading account. The
    values are updated as the algorithm runs and its keys remain unchanged.
    If connected to a broker, one can update these values with the trading
    account values as reported by the broker.

    ∆ 开头的表示只是用于证券Segment
    - accrued_cash: 当前月份的累积应付利息，按照日频更新。
    - accrued_dividend: 累计分红. 指的是所有已执行但仍未支付的分红累加值
    - available_funds: 可用资金（可用于交易）。 计算方法为 equity_with_loan - initial_margin_requirement
    - ∆ buying_power: 购买力。 预估您还可以购入多少美元的股票资产。保证金账户日内最多有四倍于资金（未被占用做保证金的资金）的购买力。
                      隔夜最多有两倍的购买力。
    - cash: 现金
    - currency: 币种
    - cushion: 剩余流动性占总资产的比例，计算方法为: excess_liquidity/net_liquidation
    - ∆ day_trades_remaining: 当日剩余日内交易次数， -1 表示无限制
    - equity_with_loan:含借贷值股权(含贷款价值资产)
        - 证券 Segment: 现金价值 + 股票价值
        - 期货 Segment: 现金价值 - 维持保证金
    - excess_liquidity:剩余流动性
        - 证券 Segment: 计算方法: equity_with_loan - maintenance_margin_requirement
        - 期货 Segment: 计算方法: net_liquidation - maintenance_margin_requirement
    - ∆ gross_position_value: 证券总价值: 做多股票的价值+做空股票价值+做多期权价值+做空期权价值。以上各项取绝对值计算。
    - initial_margin_requirement: 初始保证金
    - maintenance_margin_requirement: 维持保证金
    - net_liquidation: 总资产(净清算价值)
        - 证券 Segment: 现金价值 + 股票价值 + 股票期权价值
        - 期货 Segment: 现金价值 + 盯市盈亏
    - realized_pnl: 本日已实现盈亏
    - ∆ regt_equity: 仅针对证券Segment，即根据 Regulation T 法案计算的 equity with loan（含借贷股权值）
    - ∆ regt_margin:仅针对证券Segment， 即根据 Regulation T 法案计算的 initial margin requirements（初始保证金）
    - ∆ sma: 仅针对证券Segment。隔夜风控值，每个交易日收盘前10分钟左右对账户持仓的隔夜风险进行检查，隔夜风控值需要大于0，
             否则会在收盘前对账户部分头寸强制平仓。如果交易日盘中出现隔夜风控值低于0，而时间未到收盘前10分钟，账户不会出发强平。
    - unrealized_pnl: 未实现盈亏
    - timestamp: 更新时间
    """
    def __init__(self):
        self.accrued_cash = float('inf')
        self.accrued_dividend = float('inf')
        self.available_funds = float('inf')
        self.buying_power = float('inf')
        self.cash = float('inf')
        self.currency = None
        self.cushion = float('inf')
        self.day_trades_remaining = float('inf')
        self.equity_with_loan = float('inf')
        self.excess_liquidity = float('inf')
        self.gross_position_value = float('inf')
        self.initial_margin_requirement = float('inf')
        self.maintenance_margin_requirement = float('inf')
        self.net_liquidation = float('inf')
        self.realized_pnl = float('inf')
        self.regt_equity = float('inf')
        self.regt_margin = float('inf')
        self.sma = float('inf')
        self.timestamp = None
        self.unrealized_pnl = float('inf')

    def __repr__(self):
        return "Account({0})".format(self.__dict__)


class SecuritySegment(object):
    """
    - accrued_cash: 当前月份的累积应付利息，按照日频更新。
    - accrued_dividend: 累计分红. 指的是所有已执行但仍未支付的分红累加值
    - available_funds: 可用资金（可用于交易）。 计算方法为 equity_with_loan - initial_margin_requirement
    - cash: 现金
    - equity_with_loan: 含借贷值股权(含贷款价值资产)。计算方法： 现金价值 + 股票价值
    - excess_liquidity: 剩余流动性。计算方法: equity_with_loan - maintenance_margin_requirement
    - gross_position_value: 证券总价值: 做多股票的价值+做空股票价值+做多期权价值+做空期权价值。以上各项取绝对值计算。
    - initial_margin_requirement: 初始保证金
    - leverage: gross_position_value / net_liquidation
    - maintenance_margin_requirement: 维持保证金
    - net_liquidation: 总资产(净清算价值)。 计算方法： 现金价值 + 股票价值 + 股票期权价值
    - regt_equity: 根据 Regulation T 法案计算的 equity with loan（含借贷股权值）
    - regt_margin: 根据 Regulation T 法案计算的 initial margin requirements（初始保证金）
    - sma: 隔夜风控值，每个交易日收盘前10分钟左右对账户持仓的隔夜风险进行检查，隔夜风控值需要大于0，
             否则会在收盘前对账户部分头寸强制平仓。如果交易日盘中出现隔夜风控值低于0，而时间未到收盘前10分钟，账户不会出发强平。
    - timestamp: 更新时间
    """
    def __init__(self):
        self.accrued_cash = float('inf')
        self.accrued_dividend = float('inf')
        self.available_funds = float('inf')
        self.cash = float('inf')
        self.equity_with_loan = float('inf')
        self.excess_liquidity = float('inf')
        self.gross_position_value = float('inf')
        self.initial_margin_requirement = float('inf')
        self.leverage = float('inf')
        self.maintenance_margin_requirement = float('inf')
        self.net_liquidation = float('inf')
        self.regt_equity = float('inf')
        self.regt_margin = float('inf')
        self.sma = float('inf')
        self.timestamp = None

    def __repr__(self):
        return "SecuritySegment({0})".format(self.__dict__)


class CommoditySegment(object):
    """
    - accrued_cash: 当前月份的累积应付利息，按照日频更新。
    - accrued_dividend:累计分红. 指的是所有已执行但仍未支付的分红累加值
    - available_funds: 可用资金（可用于交易）。 计算方法为 equity_with_loan - initial_margin_requirement
    - cash: 现金
    - equity_with_loan:含借贷值股权(含贷款价值资产)计算方法：现金价值 - 维持保证金
    - excess_liquidity:剩余流动性。计算方法：net_liquidation - maintenance_margin_requirement
    - initial_margin_requirement: 初始保证金
    - maintenance_margin_requirement: 维持保证金
    - net_liquidation: 总资产(净清算价值)。计算方法：现金价值 + 盯市盈亏
    - timestamp: 更新时间
    """
    def __init__(self):
        self.accrued_cash = float('inf')
        self.accrued_dividend = float('inf')
        self.available_funds = float('inf')
        self.cash = float('inf')
        self.equity_with_loan = float('inf')
        self.excess_liquidity = float('inf')
        self.initial_margin_requirement = float('inf')
        self.maintenance_margin_requirement = float('inf')
        self.net_liquidation = float('inf')
        self.timestamp = None

    def __repr__(self):
        return "CommoditySegment({0})".format(self.__dict__)


class MarketValue(object):
    """
    - currency: 货币单位
    - net_liquidation: 总资产(净清算价值)
    - cash_balance: 现金
    - stock_market_value: 股票市值
    - option_market_value: 期权市值
    - warrant_value: 窝轮市值
    - futures_pnl: 盯市盈亏
    - unrealized_pnl: 未实现盈亏
    - realized_pnl: 已实现盈亏
    - exchange_rate: 对账户主币种的汇率
    - net_dividend: 应付股息与应收股息的净值
    - timestamp: 更新时间
    """
    def __init__(self):
        self.currency = None
        self.net_liquidation = float('inf')
        self.cash_balance = float('inf')
        self.stock_market_value = float('inf')
        self.option_market_value = float('inf')
        self.warrant_value = float('inf')
        self.futures_pnl = float('inf')
        self.unrealized_pnl = float('inf')
        self.realized_pnl = float('inf')
        self.exchange_rate = float('inf')
        self.net_dividend = float('inf')
        self.timestamp = None

    def __repr__(self):
        return "MarketValue({0})".format(self.__dict__)


class PortfolioAccount(object):
    def __init__(self, account):
        self._account = account
        self._summary = Account()
        self._segments = defaultdict(Account)
        self._market_values = defaultdict(MarketValue)

    @property
    def account(self):
        """账户 id"""
        return self._account

    @property
    def summary(self):
        """
        对几个 segments 下的统计
        :return: Account 对象
        """
        return self._summary

    def segment(self, segment_name):
        if segment_name not in self._segments:  # C, S, F
            if segment_name == 'C':
                segment = CommoditySegment()
            elif segment_name == 'S':
                segment = SecuritySegment()
            elif segment_name == 'F':
                segment = Account()
            else:
                logging.info("unknown segment %s", segment_name)
                segment = Account()
            self._segments[segment_name] = segment
            return segment
        else:
            return self._segments.get(segment_name)

    @property
    def segments(self):
        """
        按照交易品种区分的账户信息
        :return: dict，分别有两个 key，'S'表示股票， 'C' 表示期货；
        """
        return self._segments

    def market_value(self, currency):
        if currency in self._market_values:
            return self._market_values.get(currency)
        else:
            market_value = MarketValue()
            self._market_values[currency] = market_value
            return market_value

    @property
    def market_values(self):
        """
        按照市场区分的市值信息
        :return: dict， 其中的 key: 'USD' 表示美国市场， 'HKD' 表示香港市场; value 是一个 MarketValue 对象
        """
        return self._market_values

    def __repr__(self):
        return "PortfolioAccount({0})".format({'account': self.account, 'summary': self.summary,
                                               'segments': self.segments, 'market_values': self.market_values})



# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""
import six
import pandas as pd

from tigeropen.common.util.string_utils import get_string
from tigeropen.common.response import TigerResponse

COLUMNS = ['contract_code', 'symbol', 'type', 'name', 'contract_month', 'multiplier', 'exchange', 'currency',
           'first_notice_date', 'last_bidding_close_time', 'last_trading_date', 'trade', 'continuous', 'min_tick']
CONTRACT_FIELD_MAPPINGS = {'contractCode': 'contract_code', 'exchangeCode': 'exchange', 'ibCode': 'symbol',
                           'contractMonth': 'contract_month', 'firstNoticeDate': 'first_notice_date',
                           'lastBiddingCloseTime': 'last_bidding_close_time', 'lastTradingDate': 'last_trading_date',
                           'minTick': 'min_tick'}


class FutureContractResponse(TigerResponse):
    def __init__(self):
        super(FutureContractResponse, self).__init__()
        self.contracts = []
        self._is_success = None

    def parse_response_content(self, response_content):
        response = super(FutureContractResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']

        if self.data:
            contract_items = []
            if isinstance(self.data, list):
                for item in self.data:
                    item_values = self.parse_contract(item)
                    contract_code = item_values.get('contract_code')
                    if contract_code is None or contract_code.endswith('main'):
                        continue
                    contract_items.append([item_values.get(tag) for tag in COLUMNS])
            elif isinstance(self.data, dict):
                item_values = self.parse_contract(self.data)
                contract_code = item_values.get('contract_code')
                if contract_code and not contract_code.endswith('main'):
                    contract_items.append([item_values.get(tag) for tag in COLUMNS])

            self.contracts = pd.DataFrame(contract_items, columns=COLUMNS)

    @staticmethod
    def parse_contract(item):
        item_values = dict()
        for key, value in item.items():
            if value is None:
                continue

            if isinstance(value, six.string_types):
                value = get_string(value)

            if key in ('lastBiddingCloseTime', 'firstNoticeDate') and (value == 0 or value == ''):
                continue
            tag = CONTRACT_FIELD_MAPPINGS[key] if key in CONTRACT_FIELD_MAPPINGS else key
            item_values[tag] = value
        return item_values



# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""

from tigeropen.common.response import TigerResponse
from tigeropen.common.util.string_utils import get_string


class SymbolNamesResponse(TigerResponse):
    def __init__(self):
        super(SymbolNamesResponse, self).__init__()
        self.symbol_names = []
        self._is_success = None

    def parse_response_content(self, response_content):
        response = super(SymbolNamesResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']

        if self.data and isinstance(self.data, list):
            self.symbol_names = [(get_string(item['symbol']), get_string(item['name'])) for item in self.data if
                                 len(item) == 2]



# -*- coding: utf-8 -*-
"""
Created on 2018/10/30

@author: gaoan
"""
import time
# from tigeropen.common.consts import QuoteKeyType
from tigeropen.push.push_client import PushClient
from tigeropen.examples.client_config import get_client_config


def on_query_subscribed_quote(symbols, focus_keys, limit, used):
    """
    查询已订阅symbol回调
    :param symbols: 订阅合约的列表
    :param focus_keys: 每个合约订阅的 key 列表
    :param limit: 当前 tigerid 可以订阅的合约数量
    :param used: 目前已订阅的合约数量
    :return:
        返回示例:
        symbols: ['00700', 'SPY'],
        focus_keys: {'00700': ['ask_size', 'latest_price', 'ask_price', 'prev_close', 'open', 'minute', 'low', 'volume',
         'bid_price', 'bid_size', 'high', 'close'], 'SPY': ['ask_size', 'latest_price', 'ask_price', 'prev_close',
         'open', 'minute', 'low', 'volume', 'bid_price', 'bid_size', 'high', 'close']},
        limit: 100,
        used: 2

    """
    print(symbols, focus_keys, limit, used)


def on_quote_changed(symbol, items, hour_trading):
    """
    行情推送回调
    :param symbol: 订阅的证券代码
    :param items: list，每个元素是一个tuple，对应订阅的字段名称和值
    :param hour_trading: 是否为盘前盘后的交易
    :return:
    items 数据示例
        [('latest_price', 339.8), ('ask_size', 42500), ('ask_price', 340.0), ('bid_size', 1400), ('bid_price', 339.8),
         ('high', 345.0), ('prev_close', 342.4), ('low', 339.2), ('open', 344.0), ('volume', 7361440),
         ('minute', {'p': 339.8, 'a': 341.084, 't': 1568098440000, 'v': 7000, 'h': 340.0, 'l': 339.8}),
         ('timestamp', '1568098469463')]
    深度行情 items 数据示例. 最多只推送前 40 档, 由 3 个等长数组构成，分别代表：price(价格)/volume(数量)/count(单量),
    相邻档位价格可能一样, 其中 count 是可选的
        [('bid_depth',
            '[[127.87,127.86,127.86,127.86,127.85,127.85,127.85,127.84,127.84,127.84,127.84,127.83,127.83, 127.83,127.83,127.83,127.82,127.81,127.8,127.8,127.8,127.8,127.8,127.8,127.8,127.79,127.79,127.78, 127.78, 127.75,127.68,127.6,127.6,127.55,127.5,127.5,127.5,127.5,127.29,127.28],
              [69,2,5,20,1,1,1,18,1,70,80,40,2,330,330,1,40,80,20,10,131,2,30,50,300,1,38,1,1,15,6,20,1,3,100,15,25,30,49,43]
             ]'),
        ('ask_depth',
            '[[127.91,127.94,127.95,127.95,127.95,127.95,127.95,127.96,127.98,127.98,127.98,127.98,127.99,127.99, 128.0,128.0,128.0,128.0,128.0,128.0,128.0,128.0,128.0,128.0,128.0,128.0,128.0,128.0,128.0,128.0,128.0, 128.0,128.0,128.0,128.0,128.0,128.0,128.0,128.0,128.0],
              [822,4,98,50,5,5,500,642,300,40,1,36,19,1,1,1,1,50,1,1,50,1,100,10,1,1,10,1,1,1,1,5,1,8,1,1,120,70,1,4]
            ]'),
        ('timestamp', 1621933454191)]

    """
    print(symbol, items, hour_trading)


def on_order_changed(account, items):
    """

    :param account:
    :param items:
    :return:
    items 数据示例:
        [('order_type', 'LMT'), ('symbol', 'ABCD'), ('order_id', 1000101463), ('sec_type', 'STK'), ('filled', 100),
        ('quantity', 100), ('segment', 'summary'), ('action', 'BUY'), ('currency', 'USD'), ('id', 173612806463631360),
        ('order_time', 1568095814556), ('time_in_force', 'DAY'), ('identifier', 'ABCD'), ('limit_price', 113.7),
        ('outside_rth', True), ('avg_fill_price', 113.7), ('trade_time', 1568095815418),
        ('status', <OrderStatus.FILLED: 'Filled'>)]
    """
    print(account, items)


def on_asset_changed(account, items):
    """

    :param account:
    :param items:
    :return:
    items 数据示例:
        [('equity_with_loan', 721583.83), ('gross_position_value', 1339641.94),
        ('excess_liquidity', 378624.18), ('available_funds', 320059.1), ('initial_margin_requirement', 497419.25),
        ('buying_power', 2293551.51), ('cash', 950059.0), ('segment', 'summary'), ('net_liquidation', 817685.72),
        ('maintenance_margin_requirement', 439061.54)]
    """
    print(account, items)


def on_position_changed(account, items):
    """

    :param account:
    :param items:
    :return:
    items 数据示例:
        [('symbol', 'ABCD'), ('market_price', 3.68525), ('market_value', 0.0), ('sec_type', 'STK'),
        ('segment', 'summary'), ('currency', 'USD'), ('quantity', 0.0), ('average_cost', 3.884548)]
    """
    print(account, items)


def subscribe_callback(destination, content):
    """
    订阅成功与否的回调
    :param destination: 订阅的类型. 有 quote, trade/asset, trade/position, trade/order
    :param content: 回调信息. 如成功 {'code': 0, 'message': 'success'}; 若失败则 code 不为0, message 为错误详情
    """
    print('subscribe:{}, callback content:{}'.format(destination, content))


def unsubscribe_callback(destination, content):
    """
    退订成功与否的回调
    :param destination: 取消订阅的类型. 有 quote, trade/asset, trade/position, trade/order
    :param content: 回调信息.
    """
    print('subscribe:{}, callback content:{}'.format(destination, content))


# def connect_callback():
#     """连接建立回调"""
#     print('connected')
#
#
# def disconnect_callback():
#     """连接断开回调. 此处利用回调进行重连"""
#     for t in range(1, 200):
#         try:
#             print('disconnected, reconnecting')
#             push_client.connect(client_config.tiger_id, client_config.private_key)
#         except:
#             print('connect failed, retry')
#             time.sleep(t)
#         else:
#             print('reconnect success')
#             return
#     print('reconnect failed, please check your network')
#

if __name__ == '__main__':
    client_config = get_client_config()
    protocol, host, port = client_config.socket_host_port
    push_client = PushClient(host, port, use_ssl=(protocol == 'ssl'))

    # 行情变动回调
    push_client.quote_changed = on_quote_changed
    # 已订阅 symbol 查询回调
    push_client.subscribed_symbols = on_query_subscribed_quote
    # 订单变动回调
    # push_client.order_changed = on_order_changed
    # 资产变动回调
    # push_client.asset_changed = on_asset_changed
    # 持仓变动回调
    # push_client.position_changed = on_position_changed

    # 订阅成功与否的回调
    push_client.subscribe_callback = subscribe_callback
    # 退订成功与否的回调
    push_client.unsubscribe_callback = unsubscribe_callback

    # 建立推送连接
    push_client.connect(client_config.tiger_id, client_config.private_key)
    # 断线重连回调
    # push_client.disconnect_callback = disconnect_callback

    # 订阅行情
    push_client.subscribe_quote(['AAPL', 'GOOG'])
    # 可以指定关注的行情key的类型, QuoteKeyType.TRADE 为成交数据, QuoteKeyType.QUOTE 为盘口数据
    # push_client.subscribe_quote(['MSFT', 'AMD'], quote_key_type=QuoteKeyType.TRADE)

    # 订阅深度行情
    push_client.subscribe_depth_quote(['AMD', 'BABA'])

    # 订阅资产变动
    push_client.subscribe_asset()
    # 订阅订单变动
    push_client.subscribe_order()
    # 订阅持仓变动
    push_client.subscribe_position()
    # 查询已订阅的 symbol
    push_client.query_subscribed_quote()

    time.sleep(600)
    push_client.disconnect()



# -*- coding: utf-8 -*-
"""
Created on 2018/9/20

@author: gaoan
"""

import json
from tigeropen.common.consts.params import *
from tigeropen.common.consts import OPEN_API_SDK_VERSION


class OpenApiRequest(object):
    def __init__(self, method, biz_model=None):
        self._method = method
        self._biz_model = biz_model

    @property
    def biz_model(self):
        return self._biz_model

    @biz_model.setter
    def biz_model(self, value):
        self._biz_model = value

    def get_params(self):
        params = dict()
        params[P_METHOD] = self._method
        params[P_VERSION] = OPEN_API_SDK_VERSION

        if self.biz_model:
            params[P_BIZ_CONTENT] = json.dumps(obj=self.biz_model.to_openapi_dict(), ensure_ascii=False, sort_keys=True,
                                               separators=(',', ':'))

        return params



# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""
import json
import six
from tigeropen.common.response import TigerResponse
from tigeropen.common.util.string_utils import get_string
from tigeropen.trade.domain.contract import Contract
from tigeropen.trade.response import CONTRACT_FIELDS

CONTRACT_FIELD_MAPPINGS = {'secType': 'sec_type', 'localSymbol': 'local_symbol', 'originSymbol': 'origin_symbol',
                           'conid': 'contract_id', 'contractId': 'contract_id',
                           'shortMargin': 'short_margin', 'shortFeeRate': 'short_fee_rate',
                           'longInitialMargin': 'long_initial_margin', 'contractMonth': 'contract_month',
                           'longMaintenanceMargin': 'long_maintenance_margin', 'primaryExchange': 'primary_exchange',
                           'tradingClass': 'trading_class', 'lastTradingDate': 'last_trading_date',
                           'minTick': 'min_tick', 'firstNoticeDate': 'first_notice_date',
                           'lastBiddingCloseTime': 'last_bidding_close_time'}


class ContractsResponse(TigerResponse):
    def __init__(self):
        super(ContractsResponse, self).__init__()
        self.contracts = []
        self._is_success = None
    
    def parse_response_content(self, response_content):
        response = super(ContractsResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']
        
        if self.data:
            data_json = self.data if isinstance(self.data, dict) else json.loads(self.data)
            if 'items' in data_json:
                for item in data_json['items']:
                    contract_fields = {}
                    for key, value in item.items():
                        if value is None:
                            continue
                        if isinstance(value, six.string_types):
                            value = get_string(value)
                        tag = CONTRACT_FIELD_MAPPINGS[key] if key in CONTRACT_FIELD_MAPPINGS else key
                        if tag in CONTRACT_FIELDS:
                            contract_fields[tag] = value

                    contract_id = contract_fields.get('contract_id')
                    symbol = contract_fields.get('symbol')
                    currency = contract_fields.get('currency')
                    sec_type = contract_fields.get('sec_type')
                    exchange = contract_fields.get('exchange')
                    origin_symbol = contract_fields.get('origin_symbol')
                    local_symbol = contract_fields.get('local_symbol')
                    expiry = contract_fields.get('expiry')
                    strike = contract_fields.get('strike')
                    put_call = contract_fields.get('right')
                    multiplier = contract_fields.get('multiplier')
                    name = contract_fields.get('name')
                    short_margin = contract_fields.get('short_margin')
                    short_fee_rate = contract_fields.get('short_fee_rate')
                    shortable = contract_fields.get('shortable')
                    long_initial_margin = contract_fields.get('long_initial_margin')
                    long_maintenance_margin = contract_fields.get('long_maintenance_margin')
                    contract_month = contract_fields.get('contract_month')
                    identifier = contract_fields.get('identifier')
                    primary_exchange = contract_fields.get('primary_exchange')
                    market = contract_fields.get('market')
                    min_tick = contract_fields.get('min_tick')
                    trading_class = contract_fields.get('trading_class')
                    status = contract_fields.get('status')
                    continuous = contract_fields.get('continuous')
                    trade = contract_fields.get('trade')
                    last_trading_date = contract_fields.get('last_trading_date')
                    first_notice_date = contract_fields.get('first_notice_date')
                    last_bidding_close_time = contract_fields.get('last_bidding_close_time')
                    contract = Contract(symbol, currency, contract_id=contract_id, sec_type=sec_type, exchange=exchange,
                                        origin_symbol=origin_symbol, local_symbol=local_symbol, expiry=expiry,
                                        strike=strike, put_call=put_call, multiplier=multiplier, name=name,
                                        short_margin=short_margin, short_fee_rate=short_fee_rate, shortable=shortable,
                                        long_initial_margin=long_initial_margin, contract_month=contract_month,
                                        long_maintenance_margin=long_maintenance_margin, identifier=identifier,
                                        primary_exchange=primary_exchange, market=market, min_tick=min_tick,
                                        trading_class=trading_class, status=status, continuous=continuous, trade=trade,
                                        last_trading_date=last_trading_date, first_notice_date=first_notice_date,
                                        last_bidding_close_time=last_bidding_close_time)
                    self.contracts.append(contract)



# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""


class Bar(object):
    def __init__(self):
        self.time = None
        self.open = None
        self.high = None
        self.low = None
        self.close = None
        self.volume = None

    def __repr__(self):
        """
        String representation for this object.
        """
        return "Bar(%s)" % self.__dict__



import logging
import pandas as pd
import talib as ta

from datetime import datetime, timedelta
from time import sleep
from functools import wraps

from tigeropen.quote.quote_client import QuoteClient
from tigeropen.tiger_open_config import get_client_config
from tigeropen.trade.trade_client import TradeClient
from tigeropen.common.consts import *
from tigeropen.common.util.contract_utils import stock_contract
from tigeropen.common.util.order_utils import limit_order

PRIVATE_KEY_PATH = 'your private key path'
TIGER_ID = 'your tiger_id'
# 下单使用的账户
ACCOUNT = 'your account'
# 查询订单成交的重试次数
MAX_RETRY = 30
# 日志文件的路径
LOG_PATH = 'sp500.log'

# sp500 的成分股
UNIVERSE = ['MMM', 'ABT', 'ABBV', 'ACN', 'ATVI', 'AYI', 'ADBE', 'AMD', 'AAP', 'AES', 'AET', 'AMG', 'AFL', 'A', 'APD',
            'AKAM', 'ALK', 'ALB', 'ARE', 'ALXN', 'ALGN', 'ALLE', 'AGN', 'ADS', 'LNT', 'ALL', 'GOOGL', 'GOOG', 'MO',
            'AMZN', 'AEE', 'AAL', 'AEP', 'AXP', 'AIG', 'AMT', 'AWK', 'AMP', 'ABC', 'AME', 'AMGN', 'APH', 'APC', 'ADI',
            'ANDV', 'ANSS', 'ANTM', 'AON', 'AOS', 'APA', 'AIV', 'AAPL', 'AMAT', 'APTV', 'ADM', 'ARNC', 'AJG', 'AIZ',
            'T', 'ADSK', 'ADP', 'AZO', 'AVB', 'AVY', 'BHGE', 'BLL', 'BAC', 'BK', 'BAX', 'BBT', 'BDX', 'BRK.B', 'BBY',
            'BIIB', 'BLK', 'HRB', 'BA', 'BKNG', 'BWA', 'BXP', 'BSX', 'BHF', 'BMY', 'AVGO', 'BF.B', 'CHRW', 'CA', 'COG',
            'CDNS', 'CPB', 'COF', 'CAH', 'KMX', 'CCL', 'CAT', 'CBOE', 'CBRE', 'CBS', 'CELG', 'CNC', 'CNP', 'CTL',
            'CERN', 'CF', 'SCHW', 'CHTR', 'CVX', 'CMG', 'CB', 'CHD', 'CI', 'XEC', 'CINF', 'CTAS', 'CSCO', 'C', 'CFG',
            'CTXS', 'CLX', 'CME', 'CMS', 'KO', 'CTSH', 'CL', 'CMCSA', 'CMA', 'CAG', 'CXO', 'COP', 'ED', 'STZ', 'COO',
            'GLW', 'COST', 'COTY', 'CCI', 'CSX', 'CMI', 'CVS', 'DHI', 'DHR', 'DRI', 'DVA', 'DE', 'DAL', 'XRAY', 'DVN',
            'DLR', 'DFS', 'DISCA', 'DISCK', 'DISH', 'DG', 'DLTR', 'D', 'DOV', 'DWDP', 'DPS', 'DTE', 'DRE', 'DUK',
            'DXC', 'ETFC', 'EMN', 'ETN', 'EBAY', 'ECL', 'EIX', 'EW', 'EA', 'EMR', 'ETR', 'EVHC', 'EOG', 'EQT', 'EFX',
            'EQIX', 'EQR', 'ESS', 'EL', 'ES', 'RE', 'EXC', 'EXPE', 'EXPD', 'ESRX', 'EXR', 'XOM', 'FFIV', 'FB', 'FAST',
            'FRT', 'FDX', 'FIS', 'FITB', 'FE', 'FISV', 'FLIR', 'FLS', 'FLR', 'FMC', 'FL', 'F', 'FTV', 'FBHS', 'BEN',
            'FCX', 'GPS', 'GRMN', 'IT', 'GD', 'GE', 'GGP', 'GIS', 'GM', 'GPC', 'GILD', 'GPN', 'GS', 'GT', 'GWW', 'HAL',
            'HBI', 'HOG', 'HRS', 'HIG', 'HAS', 'HCA', 'HCP', 'HP', 'HSIC', 'HSY', 'HES', 'HPE', 'HLT', 'HOLX', 'HD',
            'HON', 'HRL', 'HST', 'HPQ', 'HUM', 'HBAN', 'HII', 'IDXX', 'INFO', 'ITW', 'ILMN', 'IR', 'INTC', 'ICE',
            'IBM', 'INCY', 'IP', 'IPG', 'IFF', 'INTU', 'ISRG', 'IVZ', 'IPGP', 'IQV', 'IRM', 'JEC', 'JBHT', 'SJM',
            'JNJ', 'JCI', 'JPM', 'JNPR', 'KSU', 'K', 'KEY', 'KMB', 'KIM', 'KMI', 'KLAC', 'KSS', 'KHC', 'KR', 'LB',
            'LLL', 'LH', 'LRCX', 'LEG', 'LEN', 'LUK', 'LLY', 'LNC', 'LKQ', 'LMT', 'L', 'LOW', 'LYB', 'MTB', 'MAC', 'M',
            'MRO', 'MPC', 'MAR', 'MMC', 'MLM', 'MAS', 'MA', 'MAT', 'MKC', 'MCD', 'MCK', 'MDT', 'MRK', 'MET', 'MTD',
            'MGM', 'KORS', 'MCHP', 'MU', 'MSFT', 'MAA', 'MHK', 'TAP', 'MDLZ', 'MON', 'MNST', 'MCO', 'MS', 'MOS', 'MSI',
            'MSCI', 'MYL', 'NDAQ', 'NOV', 'NAVI', 'NKTR', 'NTAP', 'NFLX', 'NWL', 'NFX', 'NEM', 'NWSA', 'NWS', 'NEE',
            'NLSN', 'NKE', 'NI', 'NBL', 'JWN', 'NSC', 'NTRS', 'NOC', 'NCLH', 'NRG', 'NUE', 'NVDA', 'ORLY', 'OXY',
            'OMC', 'OKE', 'ORCL', 'PCAR', 'PKG', 'PH', 'PAYX', 'PYPL', 'PNR', 'PBCT', 'PEP', 'PKI', 'PRGO', 'PFE',
            'PCG', 'PM', 'PSX', 'PNW', 'PXD', 'PNC', 'RL', 'PPG', 'PPL', 'PX', 'PFG', 'PG', 'PGR', 'PLD', 'PRU', 'PEG',
            'PSA', 'PHM', 'PVH', 'QRVO', 'PWR', 'QCOM', 'DGX', 'RRC', 'RJF', 'RTN', 'O', 'RHT', 'REG', 'REGN', 'RF',
            'RSG', 'RMD', 'RHI', 'ROK', 'COL', 'ROP', 'ROST', 'RCL', 'CRM', 'SBAC', 'SCG', 'SLB', 'STX', 'SEE', 'SRE',
            'SHW', 'SPG', 'SWKS', 'SLG', 'SNA', 'SO', 'LUV', 'SPGI', 'SWK', 'SBUX', 'STT', 'SRCL', 'SYK', 'STI',
            'SIVB', 'SYMC', 'SYF', 'SNPS', 'SYY', 'TROW', 'TTWO', 'TPR', 'TGT', 'TEL', 'FTI', 'TXN', 'TXT', 'TMO',
            'TIF', 'TWX', 'TJX', 'TMK', 'TSS', 'TSCO', 'TDG', 'TRV', 'TRIP', 'FOXA', 'FOX', 'TSN', 'UDR', 'ULTA',
            'USB', 'UAA', 'UA', 'UNP', 'UAL', 'UNH', 'UPS', 'URI', 'UTX', 'UHS', 'UNM', 'VFC', 'VLO', 'VAR', 'VTR',
            'VRSN', 'VRSK', 'VZ', 'VRTX', 'VIAB', 'V', 'VNO', 'VMC', 'WMT', 'WBA', 'DIS', 'WM', 'WAT', 'WEC', 'WFC',
            'WELL', 'WDC', 'WU', 'WRK', 'WY', 'WHR', 'WMB', 'WLTW', 'WYN', 'WYNN', 'XEL', 'XRX', 'XLNX', 'XL', 'XYL',
            'YUM', 'ZBH', 'ZION', 'ZTS']


client_config = get_client_config(private_key_path=PRIVATE_KEY_PATH, tiger_id=TIGER_ID, account=ACCOUNT,
                                  sandbox_debug=True)
quote_client = QuoteClient(client_config)
trade_client = TradeClient(client_config)


# 初始化 log 模块
logging.basicConfig(format='%(asctime)s - %(module)s - %(levelname)s - %(message)s',
                    filename=LOG_PATH,
                    filemode='a')
log = logging.getLogger(__name__)
log.setLevel(logging.INFO)


def recorder(func):
    @wraps(func)
    def do_log(*args, **kwargs):
        try:
            log.info('func name is %s, args are %s, kwars are %s' % (func.__name__, args, kwargs))
            return func(*args, **kwargs)
        except Exception as e:
            log.warning(e, exc_info=True)
    return do_log


@recorder
def get_daily_close(symbols):
    """获取日级收盘价
    :param symbols:证券代码列表
    """
    i = 0
    price = pd.DataFrame()
    while i <= len(symbols):
        batch = symbols[i:i+50]
        i += 50
        temp = quote_client.get_bars(symbols=batch)
        temp['time'] = temp['time'].map(lambda x: datetime.fromtimestamp(x/1000))
        price = price.append(temp)
        log.info('batch number is %s' % i)

    return price


@recorder
def calculate_indicators(price):
    """计算用于交易的指标: 昨收价对于 EMA 的偏离值
    :return:Series，index 是 symbol， value 是指标的值
    """
    # 先处理数据结构, index 是时间， column 是股票代码
    price = price[['symbol', 'time', 'close']]
    price.set_index('time', inplace=True)
    close = price.pivot(columns='symbol')
    close.columns = close.columns.droplevel()
    # 计算调仓使用的技术指标
    ema = close.apply(lambda x: ta.EMA(x.values, timeperiod=10))
    indicator = 1 - (ema.iloc[-1, :]/close.iloc[-1, :])
    return indicator


@recorder
def get_optimal_portfolio(indicator, holding_num):
    """计算市值加权的目标调仓权重
    """

    target_stock = indicator.sort_values()[:holding_num]
    log.info('target stock is %s, target num is %s' % (target_stock, holding_num))
    market_cap = quote_client.get_financial_daily(symbols=list(target_stock.index),
                                                  market=Market.US,
                                                  fields=[Valuation.market_capitalization],
                                                  begin_date=(datetime.now()-timedelta(days=4)).timestamp()*1000,
                                                  end_date=datetime.now().timestamp()*1000)

    market_cap = market_cap.sort_values('date', ascending=False).drop_duplicates(subset='symbol', keep='first')
    log.info('market cap is %s' % market_cap)
    market_cap.set_index('symbol', inplace=True)
    market_cap = market_cap['value']
    return market_cap/market_cap.sum()


@recorder
def batch_get_stock_briefs(symbols):
    i = 0
    briefs = pd.DataFrame()
    while i <= len(symbols):
        batch = symbols[i:i+50]
        i += 50
        temp = quote_client.get_stock_briefs(symbols=batch)
        briefs = briefs.append(temp)
        log.info('batch number is %s' % i)

    return briefs


@recorder
def execute(target_weight):
    """调仓，并返回结果
    """
    positions = trade_client.get_positions(account=ACCOUNT, market=Market.US)
    # 当前持仓
    current_holdings = {}
    # 卖单
    sell_orders = {}
    # 买单
    buy_orders = {}

    for i in positions:
        current_holdings.update({i.contract.symbol: i.quantity})
    if len(current_holdings.keys()) != 0:
        # 先卖出当前持仓中不在目标调仓列表中的股票
        briefs = batch_get_stock_briefs(symbols=list(current_holdings.keys()))
        briefs.set_index('symbol', inplace=True)
        latest_price = briefs.latest_price
        current_holdings = pd.Series(current_holdings)

        liquidation_list = set(current_holdings.index) - set(target_weight.index)
        log.info('executing sell orders ,latest_price are %s' % latest_price)

        for symbol in liquidation_list:
            contract = stock_contract(symbol, currency='USD')
            quantity = current_holdings[symbol]
            limit_price = round(latest_price[symbol], 2)
            order = limit_order(account=ACCOUNT,
                                contract=contract,
                                action='SELL' if quantity > 0 else 'BUY',
                                quantity=abs(int(quantity)),
                                limit_price=limit_price)
            try:
                trade_client.place_order(order)
                sell_orders.update({order.id: order})
            except Exception:
                log.error('Faild to get result, contract: %s, limit_price: %s' % (contract, limit_price), exc_info=True)
        # 轮询卖单的执行情况
        retry_time = 0
        while retry_time < MAX_RETRY and len(sell_orders.keys()) != 0:
            filled_orders = trade_client.get_filled_orders(account=ACCOUNT,
                                                           sec_type=SecurityType.STK,
                                                           market=Market.US,
                                                           start_time=(datetime.now()-timedelta(days=1)).timestamp()*1000,
                                                           end_time=datetime.now().timestamp()*1000)
            for order in filled_orders:
                sell_orders.pop(order.id, None)

            retry_time += 1
            sleep(20)
            log.info('retry time %s, remaining orders %s' % (retry_time, len(sell_orders.keys())))
        else:
            # 取消所有open 状态的订单
            for id in sell_orders.keys():
                trade_client.cancel_order(account=ACCOUNT, id=id)
        sleep(2)

    # 下买单
    # 先获取 security Segment 的账户信息（标准与模拟账户都类似， 环球账户不同）
    assets = trade_client.get_assets(account=ACCOUNT)[0].segments['S']
    # 按照当前的可用现金下单（无杠杆）
    cash = assets.cash
    if cash < 0:
        return
    target_value = cash * target_weight
    briefs = batch_get_stock_briefs(symbols=list(target_value.index))
    briefs.set_index('symbol', inplace=True)
    latest_price = briefs.latest_price
    log.info('cash is %s, target_value is %s' % (cash, target_value))
    for symbol in target_value.index:
        contract = stock_contract(symbol, 'USD')
        limit_price = round(latest_price[symbol], 2)  # 不同合约对下单价格的变动单位不同
        order = limit_order(account=ACCOUNT,
                            contract=contract,
                            action='BUY',
                            quantity=int(target_value[symbol]/latest_price[symbol]),
                            limit_price=limit_price)
        try:
            trade_client.place_order(order)
            buy_orders.update({order.id: order})
        except Exception:
            log.error('Faild to get result, contract: %s, limit_price: %s' % (contract, limit_price), exc_info=True)

    # 检查更新订单的状态
    for  id in buy_orders.keys():
        order = trade_client.get_order(account=ACCOUNT, id=id)
        buy_orders[id] = order
    log.info('buy order status %s' % buy_orders)


def run():
    price = get_daily_close(symbols=UNIVERSE)
    indicator = calculate_indicators(price)
    optimal_portfolio = get_optimal_portfolio(indicator, 10)
    execute(optimal_portfolio)


if __name__ == '__main__':
    run()



import pytz
from datetime import datetime, timedelta
import pandas as pd
import numpy as np
from tigeropen.common.consts import BarPeriod
from .client import quote_client
from .setting import MARKET

COLUMNS = ['symbol', 'time', 'open', 'high', 'low', 'close', 'volume']


class BarManager(object):
    def __init__(self):
        self.curr_bar = None
        self.data_bar = pd.DataFrame(columns=COLUMNS)
        self.last_minute = None
        self.last_volume = 0.0
        self.last_timestamp = None


class BarUtil(object):
    def __init__(self):
        self.bar_manager = {}
        self.timezone = pytz.timezone(MARKET.TIMEZONE)

        if MARKET.LUNCH_BREAK_START_TIME:
            today = datetime.now(tz=self.timezone).date()
            self.lunch_break_start = datetime.combine(today, datetime.strptime(MARKET.LUNCH_BREAK_START_TIME, '%H:%M:%S').time()).astimezone(self.timezone)
            self.lunch_break_end = datetime.combine(today, datetime.strptime(MARKET.LUNCH_BREAK_END_TIME, '%H:%M:%S').time()).astimezone(self.timezone)
        else:
            self.lunch_break_start = None
            self.lunch_break_end = None

        if MARKET.name == 'CN' or MARKET.name == 'SH' or MARKET.name == 'SZ':
            self.bar_func = self.get_bar_arr
        else:
            self.bar_func = quote_client.get_bars

    def on_data(self, symbol, items):
        """
        1.according to the market data time, market ticker data is splitted into minute bar
        2.the last bar is stored into data_bar after the current minute changes to the next minute.
          e.g. 9:30:59 -> 9:31:01
        3.user could change the split method to get the latest minute to shorten the delay.
          e.g. change when second reaches to 55, 9:30:54 -> 9:30:56
        """
        price_dict = dict(items)
        latest_price = price_dict.get('latest_price')
        volume = price_dict.get('volume')
        timestamp = price_dict.get('latest_time')

        curr_datetime_minute = (pd.to_datetime(timestamp, unit='ms').tz_localize('utc').tz_convert(MARKET.TIMEZONE)).replace(second=0, microsecond=0)

        if self.lunch_break_start and self.lunch_break_start < curr_datetime_minute < self.lunch_break_end:
            return

        curr_bar_manager = self.bar_manager.get(symbol)
        if latest_price:
            if curr_bar_manager:
                start_idx = curr_bar_manager.last_minute

                if curr_datetime_minute > start_idx:
                    if (curr_datetime_minute >= self.lunch_break_end) and (start_idx <= self.lunch_break_start):
                        delta1 = self.lunch_break_start - start_idx
                        delta2 = curr_datetime_minute - self.lunch_break_end

                        for i in range(delta1.seconds // 60):
                            curr_bar = curr_bar_manager.curr_bar.copy()
                            idx_datetime = start_idx + timedelta(minutes=i)
                            curr_bar.time = int(idx_datetime.timestamp() * 1000)
                            curr_bar_manager.data_bar = curr_bar_manager.data_bar.append(curr_bar, ignore_index=True)

                        for i in range(delta2.seconds // 60):
                            curr_bar = curr_bar_manager.curr_bar.copy()
                            idx_datetime = self.lunch_break_end + timedelta(minutes=i)
                            curr_bar.time = int(idx_datetime.timestamp() * 1000)
                            curr_bar_manager.data_bar = curr_bar_manager.data_bar.append(curr_bar, ignore_index=True)
                    else:
                        delta = curr_datetime_minute - start_idx
                        for i in range(delta.seconds // 60):
                            curr_bar = curr_bar_manager.curr_bar.copy()
                            idx_datetime = start_idx + timedelta(minutes=i)
                            curr_bar.time = int(idx_datetime.timestamp() * 1000)

                            curr_bar_manager.data_bar = curr_bar_manager.data_bar.append(curr_bar, ignore_index=True)

                    curr_bar_manager.last_timestamp = timestamp

                    # update latest
                    curr_bar_manager.curr_bar = pd.Series([symbol, int(curr_datetime_minute.timestamp() * 1000),
                                                           latest_price, latest_price, latest_price,
                                                           latest_price, volume - curr_bar_manager.last_volume], index=COLUMNS)
                    curr_bar_manager.last_minute = curr_datetime_minute
                    curr_bar_manager.last_volume = volume
                elif curr_datetime_minute == start_idx:
                    if timestamp > curr_bar_manager.last_timestamp:
                        curr_bar_manager.curr_bar.high = max(curr_bar_manager.curr_bar.high, latest_price)
                        curr_bar_manager.curr_bar.low = min(curr_bar_manager.curr_bar.low, latest_price)
                        curr_bar_manager.curr_bar.close = latest_price
                        curr_bar_manager.curr_bar.volume = volume - curr_bar_manager.last_volume
                        curr_bar_manager.last_timestamp = timestamp
            else:
                curr_bar_manager = BarManager()
                self.bar_manager[symbol] = curr_bar_manager
                curr_bar_manager.curr_bar = pd.Series([symbol, int(curr_datetime_minute.timestamp() * 1000),
                                                       latest_price, latest_price, latest_price,
                                                       latest_price, volume], index=COLUMNS)
                curr_bar_manager.last_minute = curr_datetime_minute
                curr_bar_manager.last_volume = volume
                curr_bar_manager.last_timestamp = timestamp

    def get_bar_arr(self, symbols, period, limit, end_time):
        if period == BarPeriod.DAY:
            return quote_client.get_bars(symbols=symbols, period=period, limit=limit, end_time=end_time)
        else:
            frames = []
            for symbol in symbols:
                curr_bar_manager = self.bar_manager.get(symbol)
                if curr_bar_manager:
                    df = curr_bar_manager.data_bar
                    curr_df = df[(df['time'] < end_time)].tail(limit)
                    frames.append(curr_df)
                else:
                    frames.append(pd.DataFrame(columns=COLUMNS))

            return pd.concat(frames)

    def get_bars(self, symbols, period, limit, end_time):
        return self.bar_func(symbols=symbols, period=period, limit=limit, end_time=end_time)


minute_bar_util = BarUtil()


class StockQuote:
    def __init__(self, time):
        self.time = time

    @staticmethod
    def get_bars(symbol, period, limit, end_ts):
        return minute_bar_util.get_bars(symbols=[symbol], period=period, limit=limit, end_time=end_ts)

    def current(self, assets, fields):

        if isinstance(assets, list):
            symbols = assets
        else:
            symbols = [assets]

        end_ts = int(self.time.timestamp() * 1000)
        bars = list()
        for symbol in symbols:
            bar = self.get_bars(symbol, BarPeriod.ONE_MINUTE, 1, end_ts)
            bars.append(bar)
        data = pd.concat(bars, ignore_index=True)
        if data.empty:
            return np.nan
        if isinstance(assets, list) and isinstance(fields, list):
            return data[fields].set_index(pd.Series(assets))
        elif isinstance(assets, list):
            return data.set_index(pd.Series(assets))[fields]
        elif isinstance(fields, list):
            return data[fields].transpose()[0]
        else:
            return data[fields].iloc[0]

    def history(self, assets, fields, bar_count, frequency='1m'):
        if frequency == '1m':
            period = BarPeriod.ONE_MINUTE
        else:
            period = BarPeriod.DAY
        end_dt = self.time
        end_ts = int(end_dt.timestamp() * 1000)

        if isinstance(assets, list):
            symbols = assets
        else:
            symbols = [assets]

        bars = list()
        for symbol in symbols:
            bar = self.get_bars(symbol, period, bar_count, end_ts)
            bars.append(bar)
        data = pd.concat(bars, ignore_index=True)
        if data.empty:
            return np.nan
        data.time = pd.to_datetime(data.time, unit='ms').dt.tz_localize('utc').dt.tz_convert(MARKET.TIMEZONE)
        if isinstance(assets, list) and isinstance(fields, list):
            return data.set_index(['time', 'symbol']).to_panel()[fields]
        elif isinstance(assets, list):
            result = data.pivot(index='time', columns='symbol')[fields]
            result.index.name = None
            result.columns.name = None
            return result
        elif isinstance(fields, list):
            result = data.set_index('time')[fields]
            result.index.name = None
            return result
        else:
            result = data.set_index('time')[fields]
            result.index.name = None
            return result



# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""
import pandas as pd

from tigeropen.common.response import TigerResponse

COLUMNS = ['start', 'end', 'trading', 'bidding', 'zone']


class FutureTradingTimesResponse(TigerResponse):
    def __init__(self):
        super(FutureTradingTimesResponse, self).__init__()
        self.trading_times = []
        self._is_success = None

    def parse_response_content(self, response_content):
        response = super(FutureTradingTimesResponse, self).parse_response_content(response_content)
        if 'is_success' in response:
            self._is_success = response['is_success']

        if self.data:
            zone = self.data.get('timeSection')
            bidding_times = self.data.get('biddingTimes')
            trading_times = self.data.get('tradingTimes')

            time_items = []
            if bidding_times:
                for item in bidding_times:
                    start = item.get('start')
                    end = item.get('end')
                    time_items.append([start, end, False, True, zone])
            if trading_times:
                for item in trading_times:
                    start = item.get('start')
                    end = item.get('end')
                    time_items.append([start, end, True, False, zone])

            self.trading_times = pd.DataFrame(time_items, columns=COLUMNS)



import os
import time
import pytz
import logbook
import sys
from datetime import datetime, timedelta

ENGINE_PATH = os.path.dirname(os.path.realpath(__file__))
RUN_PATH = os.path.abspath(os.path.join(ENGINE_PATH, os.pardir, os.pardir, os.pardir))
sys.path.insert(0, RUN_PATH)

from tigeropen.trade.domain.position import Position
from tigeropen.examples.tinyquant.data import minute_bar_util, StockQuote as Data
from tigeropen.examples.tinyquant.strategy import CompatibleStrategy, TickStrategy
from tigeropen.examples.tinyquant.client import client_config, push_client, trade_client, quote_client, \
    global_context
import tigeropen.examples.tinyquant.setting as setting


logbook.set_datetime_format("local")
logbook.StreamHandler(sys.stdout,
                      format_string='[{record.time:%Y-%m-%d %H:%M:%S.%f%z}] {record.level_name}: {record.channel}:{record.lineno}: {record.message}').push_application()
logger = logbook.Logger('[engine]')


# ============= set symbols ===================
subscribe_symbols = global_context.subscribed_symbols


timezone = pytz.timezone(setting.MARKET.TIMEZONE)
IS_EVENT_TRIGGER = setting.EVENT_TRIGGER


def on_query_subscribed_quote(symbols, focus_keys, limit, used):
    logger.debug('{symbols}, {focus_keys}, {limit}, {used}'.format(
        symbols=symbols, focus_keys=focus_keys, limit=limit, used=used))
    unsubscribe_symbols = set(symbols) - subscribe_symbols
    if unsubscribe_symbols:
        logger.debug(unsubscribe_symbols)
        # push_client.unsubscribe_quote(symbols=unsubscribe_symbols)

    if IS_EVENT_TRIGGER:
        push_client.quote_changed = on_quote_changed_event_trigger
    else:
        push_client.quote_changed = on_quote_changed
    push_client.subscribe_quote(subscribe_symbols)


def subscribe_initialize():
    push_client.subscribed_symbols = on_query_subscribed_quote
    push_client.query_subscribed_quote()

    push_client.asset_changed = on_asset_changed
    push_client.position_changed = on_position_changed
    push_client.order_changed = on_order_changed

    push_client.subscribe_asset()
    push_client.subscribe_position()
    push_client.subscribe_order()


def unsubscribe_process():
    push_client.unsubscribe_order()
    push_client.unsubscribe_position()
    push_client.unsubscribe_asset()
    push_client.unsubscribe_quote(symbols=subscribe_symbols)
    push_client.disconnect()


def add_order(order):
    global_context.order_manager[order.order_id] = order
    global_context.active_order_manager[order.order_id] = global_context.order_manager[order.order_id]


# ============= strategy with event trigger ===================
def on_quote_changed_event_trigger(symbol, items, hour_trading):
    if hour_trading:
        return
    if symbol in subscribe_symbols:
        if setting.MARKET.name != 'HK' and setting.MARKET.name != 'US':
            minute_bar_util.on_data(symbol, items)


# ============= subscribe info ===================
def on_asset_changed(account, items):
    # DU575569 [('equity_with_loan', 776871.76), ('gross_position_value', 349025.33), ('excess_liquidity', 653692.01),
    # ('available_funds', 648601.77), ('initial_margin_requirement', 128269.99), ('buying_power', 4324011.82),
    # ('cash', 476021.26), ('net_liquidation', 776871.76), ('maintenance_margin_requirement', 123179.75)]
    logger.debug('{account}, {items}'.format(account=account, items=items))
    ret_account = dict(items)
    curr_account = global_context.asset_manager
    if curr_account:
        curr_account.summary.equity_with_loan = ret_account.get('equity_with_loan')
        curr_account.summary.gross_position_value = ret_account.get('gross_position_value')
        curr_account.summary.excess_liquidity = ret_account.get('excess_liquidity')
        curr_account.summary.available_funds = ret_account.get('available_funds')
        curr_account.summary.initial_margin_requirement = ret_account.get('initial_margin_requirement')
        curr_account.summary.buying_power = ret_account.get('buying_power')
        curr_account.summary.cash = ret_account.get('cash')
        curr_account.summary.net_liquidation = ret_account.get('net_liquidation')
        curr_account.summary.maintenance_margin_requirement = ret_account.get('maintenance_margin_requirement')


def on_position_changed(account, items):
    # DU575569 [('market_price', 23.37599945), ('market_value', 9350.4), ('sec_type', 'STK'),
    # ('origin_symbol', '600053'), ('unrealized_pnl', -101.62), ('quantity', 400.0), ('average_cost', 23.630052)]
    logger.debug('{account}, {items}'.format(account=account, items=items))
    ret_position = dict(items)
    symbol = ret_position.get('origin_symbol')
    if symbol in global_context.contract_map:
        curr_position = global_context.position_manager.get(symbol)
        if curr_position:
            curr_position.quantity = ret_position.get('quantity')
            curr_position.average_cost = ret_position.get('average_cost')
            curr_position.market_price = ret_position.get('market_price')
            curr_position.market_value = ret_position.get('market_value')
            curr_position.unrealized_pnl = ret_position.get('unrealized_pnl')
            curr_position.realized_pnl = ret_position.get('realized_pnl')
        else:
            global_context.position_manager[symbol] = Position(account=global_context.account,
                                                               contract=global_context.contract_map[symbol],
                                                               quantity=ret_position.get('quantity'),
                                                               average_cost=ret_position.get('average_cost'),
                                                               market_price=ret_position.get('market_price'),
                                                               market_value=ret_position.get('market_value'),
                                                               realized_pnl=ret_position.get('realized_pnl'),
                                                               unrealized_pnl=ret_position.get('unrealized_pnl'))


def on_order_changed(account, items):
    logger.debug('{account}, {items}'.format(account=account, items=items))
    # DU575569 [('order_type', 'LMT'), ('order_id', 1000051287), ('sec_type', 'STK'),
    # ('filled', 100), ('origin_symbol', '000513'), ('quantity', 100), ('order_time', 1547620277910),
    # ('time_in_force', 'DAY'), ('limit_price', 27.81), ('last_fill_price', 0.0), ('outside_rth', True),
    # ('avg_fill_price', 26.88), ('trade_time', 1547620279026)]
    ret_order = dict(items)
    curr_order = global_context.order_manager.get(ret_order.get('order_id'))
    if curr_order:
        curr_order.filled = ret_order.get('filled')
        curr_order.order_time = ret_order.get('order_time')
        curr_order.last_fill_price = ret_order.get('last_fill_price')
        curr_order.avg_fill_price = ret_order.get('avg_fill_price')
        curr_order.trade_time = ret_order.get('trade_time')
        curr_order.reason = ret_order.get('reason')
        curr_order.realized_pnl = ret_order.get('realized_pnl')
        curr_order.commission = ret_order.get('commission')

        if curr_order.remaining <= 0:
            global_context.active_order_manager[curr_order.order_id] = None
            del global_context.active_order_manager[curr_order.order_id]


def cancel_all_open_orders():
    curr_orders = trade_client.get_open_orders()
    for curr_order in curr_orders:
        try:
            trade_client.cancel_order(order_id=curr_order.order_id)
        except Exception as e:
            logger.error(e)


def asset_initialize():
    assets = trade_client.get_assets()
    if assets:
        global_context.asset_manager = assets[0]
    else:
        raise Exception('No assets')


def position_initialize():
    positions = trade_client.get_positions()
    global_context.position_manager = {position.contract.symbol: position for position in positions}


def order_initialize():
    global_context.order_manager = {}
    global_context.active_order_manager = {}


def contract_initialize():
    for symbol in subscribe_symbols:
        curr_contracts = trade_client.get_contracts(symbol,
                                                   sec_type=global_context.security_type_map.get(symbol),
                                                   currency=global_context.currency_map.get(symbol))
        if not curr_contracts:
            raise Exception('Can not get contracts')
        global_context.contract_map[symbol] = curr_contracts[0]


if setting.EVENT_TRIGGER:
    compatible_strategy = CompatibleStrategy(push_client=push_client, trade_client=trade_client, quote_client=quote_client, context=global_context)

    def strategy_initialize():
        compatible_strategy.initialize()
        global_context.load()

    def before_trading_start(data):
        compatible_strategy.before_trading_start(data=data)

    def handle_data(data):
        compatible_strategy.handle_data(data=data)
        global_context.store()

    def run_schedule_func(data):
        if hasattr(global_context, 'schedule_function'):
            global_context.schedule_function.run(data=data)
            global_context.store()

    def dump():
        pass

else:
    strategy = TickStrategy(push_client=push_client, trade_client=trade_client, quote_client=quote_client, context=global_context)

    def on_quote_changed(symbol, items, hour_trading):
        if symbol in subscribe_symbols:
            strategy.on_ticker(symbol_str=symbol, items=items, hour_trading=hour_trading)

    def strategy_initialize():
        strategy.initialize()
        global_context.load()

    def before_trading_start(data):
        strategy.before_trading_start(data=data)

    def handle_data(data):
        pass

    def run_schedule_func(data):
        pass

    def dump():
        strategy.dump()
        global_context.store()


if __name__ == '__main__':
    asset_initialize()
    position_initialize()
    order_initialize()

    strategy_initialize()
    curr_datetime = datetime.now().astimezone(timezone)
    before_trading_start(Data(curr_datetime))

    push_client.connect(client_config.tiger_id, client_config.private_key)
    subscribe_initialize()
    contract_initialize()

    try:
        if IS_EVENT_TRIGGER:
            OPEN_TIME = setting.MARKET.OPEN_TIME
            CLOSE_TIME = setting.MARKET.CLOSE_TIME
            SYSTEM_DELAY = setting.SYSTEM_DELAY

            open_time = datetime.strptime(OPEN_TIME, '%H:%M:%S').replace(tzinfo=timezone).time().replace(
                second=SYSTEM_DELAY)
            close_time = datetime.strptime(CLOSE_TIME, '%H:%M:%S').time()

            if setting.FREQUENCY == 'minute':
                today = datetime.now(tz=timezone).date()

                curr_time = (datetime.now(tz=timezone).replace(second=SYSTEM_DELAY, microsecond=0) + timedelta(minutes=2)).time()
                next_bar_time = max(curr_time, open_time)

                if setting.MARKET.LUNCH_BREAK_START_TIME:
                    lunch_break_start = datetime.strptime(setting.MARKET.LUNCH_BREAK_START_TIME, '%H:%M:%S').time()
                    lunch_break_end = datetime.strptime(setting.MARKET.LUNCH_BREAK_END_TIME, '%H:%M:%S').time()
                    while True:
                        curr_time = datetime.now(tz=timezone).time()
                        if next_bar_time >= close_time:
                            logger.debug('event trigger finished')
                            break
                        elif lunch_break_start < next_bar_time < lunch_break_end:
                            curr_datetime = timezone.localize(datetime.combine(today, next_bar_time) - timedelta(minutes=1))
                            next_bar_time = (curr_datetime + timedelta(minutes=2)).time()
                            time.sleep(1)
                        elif next_bar_time <= curr_time:
                            curr_datetime = timezone.localize(datetime.combine(today, next_bar_time) - timedelta(minutes=1))
                            curr_data = Data(curr_datetime)

                            logger.debug(f'next bar time:{curr_datetime}')
                            run_schedule_func(curr_data)
                            handle_data(curr_data)
                            next_bar_time = (curr_datetime + timedelta(minutes=2)).time()
                        else:
                            time.sleep(1)
                else:
                    while True:
                        curr_time = datetime.now(tz=timezone).time()
                        if next_bar_time >= close_time:
                            logger.debug('event trigger finished')
                            break
                        elif next_bar_time <= curr_time:
                            curr_datetime = timezone.localize(datetime.combine(today, next_bar_time) - timedelta(minutes=1))
                            curr_data = Data(curr_datetime)
                            run_schedule_func(curr_data)
                            handle_data(curr_data)
                            next_bar_time = (curr_datetime + timedelta(minutes=2)).time()
                        else:
                            time.sleep(1)
            elif setting.FREQUENCY == 'daily':
                today = datetime.now(tz=timezone).date()
                while True:
                    curr_datetime = datetime.now(tz=timezone)
                    if curr_datetime.time() >= close_time:
                        logger.debug('daily event trigger finished')
                    elif curr_datetime.time() >= open_time:
                        curr_data = Data(curr_datetime)
                        run_schedule_func(curr_data)
                        handle_data(curr_data)
                        time.sleep(120)
                        break
                    else:
                        time.sleep(1)
        else:
            while True:
                time.sleep(600)
    except (KeyboardInterrupt, SystemExit):
        logger.debug('keyboard interrupt, system exit')
    unsubscribe_process()
    dump()



# -*- coding: utf-8 -*-
"""
Created on 2018/10/31

@author: gaoan
"""


class TradeTick(object):
    def __init__(self):
        self.index = None
        self.timestamp = None
        self.price = None
        self.size = None
        self.direction = None

    def __repr__(self):
        return "TradeTick(%s)" % self.__dict__



# 是否使用 sandbox 环境
IS_SANDBOX = True
# 私钥的路径, 如 '/Users/tiger/.ssh/rsa_private_key.pem'
PRIVATE_KEY = ''
# 申请开放平台账户返回的ID
TIGER_ID = ''
# 账户
ACCOUNT = ''
# 账户是否是模拟账户
IS_PAPER = False

class MarketSetting:
    """不同市场的设置
    """
    def __init__(self, name):
        """
        :param name: 市场名称: 美股 US; 港股 HK; A股 CN;
        """
        self.name = name.upper() if name else 'US'
        self.TIMEZONE = None
        self.CURRENCY = None

        # 市场指数的symbol
        self.INDEX = None
        # 开盘时间
        self.OPEN_TIME = None
        # 收盘时间
        self.CLOSE_TIME = None
        # 午间休市开始时间
        self.LUNCH_BREAK_START_TIME = None
        # 午间休市结束时间
        self.LUNCH_BREAK_END_TIME = None

        self.init_setting()

    def init_setting(self):
        if self.name == 'US':
            self.TIMEZONE = 'US/Eastern'
            self.OPEN_TIME = '09:30:00'
            self.CLOSE_TIME = '16:00:00'
            self.CURRENCY = 'USD'
            self.INDEX = '.INX'

        elif self.name == 'HK':
            self.TIMEZONE = 'Asia/Hong_Kong'
            self.OPEN_TIME = '09:30:00'
            self.CLOSE_TIME = '16:00:00'
            self.LUNCH_BREAK_START_TIME = '12:00:00'
            self.LUNCH_BREAK_END_TIME = '13:00:00'
            self.CURRENCY = 'HKD'
            self.INDEX = 'HSI'

        elif self.name == 'CN' or self.name == 'SH' or self.name == 'SZ':
            self.TIMEZONE = 'Asia/Shanghai'
            self.OPEN_TIME = '09:30:00'
            self.CLOSE_TIME = '15:00:00'
            self.LUNCH_BREAK_START_TIME = '11:30:00'
            self.LUNCH_BREAK_END_TIME = '13:00:00'
            self.CURRENCY = 'CNH'
            self.INDEX = '000001.SH'


# 设置市场
MARKET = MarketSetting('US')

# 是否使用时间事件驱动模式. 时间事件驱动模式兼容量化平台的策略. 若设置为 False 则使用行情驱动模式
EVENT_TRIGGER = True
# 时间事件驱动模式下策略运行频率 'minute' 或 'daily'
FREQUENCY = 'minute'

# [event trigger] this param is to balance the difference between local system time and the market data time,
# system delay time 5s
SYSTEM_DELAY = 5

# context保存路径, 如 './context.pickle'. 若为空则不保存context
CONTEXT_FILE_PATH = ''




# -*- coding: utf-8 -*-
"""
Created on 2018/9/20

@author: gaoan
"""


class ApiException(Exception):
    def __init__(self, code, msg):
        self.code = code
        self.msg = msg

    def __str__(self, *args, **kwargs):
        sb = "code=" + str(self.code) + \
             " msg=" + self.msg
        return sb


class RequestException(Exception):
    pass


class ResponseException(Exception):
    pass
